#include <QPushButton>
#include <QAction>
#include <QGroupBox>
#include <QComboBox>
#include <QLabel>
#include <QLineEdit>
#include <QMessageBox>
#include <QRegExp>
#include <QDateTimeEdit>
#include <QTime>
#include <QDate>

#include "ARsql.h"
#include "ARScarico.h"
#include "ARFinestra.h"

#include "ARArticolo.h"

ARArticolo::ARArticolo(QWidget *parent) : QWidget(parent)
 {
  mainWin=(ARFinestra *)parent;
  sql = new ARsql(this);
  infobox = new QMessageBox(tr("Informazioni"),"",QMessageBox::Question,0,0,0,this);
  warnbox = new QMessageBox(tr("Avviso"),"",QMessageBox::Warning,0,0,0,this);
  cornice = new QGroupBox(tr(""),this);
  cornice->setGeometry(2,0,650,420);

  _art        = new QLabel(tr("Art."),cornice);               _art->setGeometry       ( 10, 20, 80, 24);
  art         = new QLineEdit("",cornice);                    art->setGeometry        ( 80, 20,550, 24);
  _descr      = new QLabel(tr("Descrizione"),cornice);        _descr->setGeometry     ( 10, 50, 80, 24);
  descr       = new QLineEdit("",cornice);                    descr->setGeometry      ( 80, 50,550, 24);
  _misura     = new QLabel(tr("Misura"),cornice);             _misura->setGeometry    ( 10, 80, 80, 24);
  misura      = new QLineEdit("",cornice);                    misura->setGeometry     ( 80, 80, 60, 24);
  _prezzo_pub = new QLabel(tr("Prezzo al pubblico"),cornice); _prezzo_pub->setGeometry( 10,230, 90, 24);
  prezzo_pub  = new QLineEdit("",cornice);                    prezzo_pub->setGeometry (100,230,120, 24);
  _sconto     = new QLabel(tr("Sconto (%)"),cornice);         _sconto->setGeometry    (230,230, 80, 24);
  sconto      = new QLineEdit("",cornice);                    sconto->setGeometry     (290,230, 40, 24);
  _iva        = new QLabel(tr("IVA (%)"),cornice);            _iva->setGeometry       (340,230, 80, 24);
  iva         = new QLineEdit("",cornice);                    iva->setGeometry        (380,230, 40, 24);
  _prezzo     = new QLabel(tr("Prezzo"),cornice);             _prezzo->setGeometry    (430,230,120, 24);
  prezzo      = new QLineEdit("",cornice);                    prezzo->setGeometry     (470,230,120, 24);
  _colore       = new QLabel(tr("Colore"),cornice);   _colore->setGeometry      ( 10,110, 80, 24);
  listaColore   = new QComboBox(cornice);             listaColore->setGeometry  ( 80,110,120, 24);
  _fondo        = new QLabel(tr("Fondo"),cornice);    _fondo->setGeometry       ( 10,140, 80, 24);
  listaFondo    = new QComboBox(cornice);             listaFondo->setGeometry   ( 80,140,120, 24);
  _tacco        = new QLabel(tr("Tacco"),cornice);    _tacco->setGeometry       ( 10,170, 80, 24);
  listaTacco    = new QComboBox(cornice);             listaTacco->setGeometry   ( 80,170,120, 24);
  _stagione     = new QLabel(tr("Stagione"),cornice); _stagione->setGeometry    ( 10,200, 80, 24);
  listaStagione = new QComboBox(cornice);             listaStagione->setGeometry( 80,200,120, 24);

  _infoart = new QLabel(tr("<b>Informazioni articolo:</b>"),this); _infoart->setGeometry( 5,260,120, 24);
  info = new QLabel("",this);                                      info->setGeometry    (10,285,520,130);
  
  art->setMaxLength(255);
  descr->setMaxLength(255);
  misura->setMaxLength(5);
  prezzo->setValidator(new QDoubleValidator(-50000000.0,50000000.0,2,prezzo_pub));
  prezzo->setMaxLength(255);
  iva->setMaxLength(3);
  iva->setValidator(new QIntValidator(0,100,iva));
  sconto->setValidator(new QDoubleValidator(-9999.0,9999.0,2,sconto));
  prezzo_pub->setValidator(new QDoubleValidator(-50000000.0,50000000.0,2,prezzo_pub));
  prezzo_pub->setMaxLength(255);

  esegui = new QPushButton(QIcon("pic/floppy.png"),tr("Salva modifiche"),cornice);
  esegui->setGeometry(543,393,106,24);

  QDate cd = QDate::currentDate();
  cambia = new QDateTimeEdit(cd,this); cambia->setGeometry(270,393,152,24);
  cambia->setDisplayFormat("ddd dd-MMM-yyyy hh:mm:ss");

  cambia_data = new QPushButton(tr("Correggi data di vendita"),this);
  cambia_data->setGeometry(423,393,120,24);

  chiudi = new QPushButton(tr("X"),this); chiudi->setGeometry(634,10,15,16);

  connect(esegui,SIGNAL(clicked()),this,SLOT(addArticolo()));
  connect(cambia_data,SIGNAL(clicked()),this,SLOT(cambiaData()));
  connect(chiudi,SIGNAL(clicked()),this,SLOT(chiudiPannello()));

  infobox->setWindowModality(Qt::WindowModal);
  warnbox->setWindowModality(Qt::WindowModal);
  setGeometry(0,61,680,440);
  azione=ARARTICOLO_ADD;  
  hide();
 }

void ARArticolo::chiudiPannello()
 {
  setVisible(false);
  if(!cornice->isEnabled()) mainWin->getARScaricoP()->visMagazzino();
  mainWin->eticMnu->setEnabled(true); mainWin->stmagMnu->setEnabled(true); mainWin->ckprezMnu->setEnabled(true);
  mainWin->getARScaricoP()->show();
 }

void ARArticolo::cambiaData()
 {
  // Aggiorna data di scarico/reso
  MYSQL_RES *rs;
  MYSQL_ROW row;
  QString stato;
  QString nuova_data = cambia->date().toString("yyyy-MM-dd");
  QString nuova_ora = cambia->time().toString("hh:mm:ss");
  QString nuovo_ts;

  QDateTime *dt = new QDateTime(cambia->date(),cambia->time());
  nuovo_ts.sprintf("%u",dt->toTime_t());

  if(sql->doQuery("UPDATE articoli SET data_sr='"+nuova_data+"', ora_sr='"+nuova_ora+"', ts_sr='"+nuovo_ts+"' WHERE articoloID='"+articoloID+"'"))
   {
    warnbox->setText(tr("Data modificata e salvata!"));
    warnbox->exec();
    rs=sql->doQueryRS("SELECT * FROM articoli WHERE articoloID='"+articoloID+"'");
    if(rs)
     {
      row=mysql_fetch_row(rs);
      stato = row[23];
      mysql_free_result(rs);
     }
    setForm(articoloID,stato);
   }
 }

void ARArticolo::addArticolo()
 {
  QString colore,coloreID,fondo,fondoID,tacco,taccoID,stagione,stagioneID;
  QString art=quote(this->art->text());
  QString descr=quote(this->descr->text());
  QString misura=quote(this->misura->text());
  QString prezzo=quote(this->prezzo->text());
  QString iva=quote(this->iva->text());
  QString sconto=quote(this->sconto->text());
  QString prezzo_pub=quote(this->prezzo_pub->text());
  QString ts=sql->timeStamp();
  QString data=sql->data(ts);
  QString ora=sql->ora(ts);
  if(listaColore->isEnabled())
   {
    coloreID=listaColore->itemData(listaColore->currentIndex()).toString();
    colore=listaColore->itemText(listaColore->currentIndex());
   }
  else { coloreID="0"; colore=""; }
  if(listaFondo->isEnabled())
   {
    fondoID=listaFondo->itemData(listaFondo->currentIndex()).toString();
    fondo=listaFondo->itemText(listaFondo->currentIndex());
   }
  else { fondoID="0"; fondo=""; }
  if(listaTacco->isEnabled())
   {
    taccoID=listaTacco->itemData(listaTacco->currentIndex()).toString();
    tacco=listaTacco->itemText(listaTacco->currentIndex());
   }
  else { taccoID="0"; tacco=""; }
  if(listaStagione->isEnabled())
   {
    stagioneID=listaStagione->itemData(listaStagione->currentIndex()).toString();
    stagione=listaStagione->itemText(listaStagione->currentIndex());
   }
  else { stagioneID="0"; stagione=""; }

/* 	if(isAzione(ARARTICOLO_ADD))
 	 {
 	  if(!checkForm())
 	   {
 	   	if(sql->doQuery("INSERT INTO articoli VALUES('0','"+art+"','"+descr+"','"+misura+"','"+prezzo+"','"+iva+"','"+prezzo_pub+"','"+data+"','"+ora+"','"+ts+"')"))
       {
        infobox->setText(tr("Nuovo articolo inserito"));
        infobox->exec();
        clearForm();
 	     }
 	   }
 	 }*/
 	if(isAzione(ARARTICOLO_EDIT))
 	 {
 	  if(!checkForm())
 	   {
 	 	 	MYSQL_RES *rs;
 	    MYSQL_ROW row;

      // Entro nella sezione critica
      mainWin->sql->connetti();
      if(!mainWin->sql->bloccaPer(sql->getDbName()+".venditacassa","0")) // .venditacassa_n
       {
        warnbox->setText(tr("Un'altro utente sta usando gli articoli,\nnon posso modificare l'articolo adesso. Riprova pi tardi."));
 	      warnbox->exec();
        mainWin->sql->disconnetti();
        return;
       }
      mainWin->sql->disconnetti();
      // Entro nella sezione critica
 	   	mainWin->sql->connetti();
 	   	mainWin->sql->doQueryPer("LOCK TABLES articoli WRITE, colori WRITE, fondi WRITE, tacchi WRITE, stagioni WRITE");
 	   	rs=mainWin->sql->doQueryRSPer("SELECT descrizione FROM colori WHERE coloreID='"+coloreID+"'");
 	   	if(rs)
 	   	 {
 	   	  if(mysql_num_rows(rs)==0) { coloreID="0"; colore=""; }
 	   	  else { row=mysql_fetch_row(rs); colore=row[0]; }
 	   	  mysql_free_result(rs);
 	   	 }
 	   	else { coloreID="0"; colore=""; }
 	   	rs=mainWin->sql->doQueryRSPer("SELECT descrizione FROM fondi WHERE fondoID='"+fondoID+"'");
 	   	if(rs)
 	   	 {
   	   	if(mysql_num_rows(rs)==0) { fondoID="0"; fondo=""; }
 	    	else { row=mysql_fetch_row(rs); fondo=row[0]; }
 	    	mysql_free_result(rs);
 	     }
 	    else { fondoID="0"; fondo=""; }
      rs=mainWin->sql->doQueryRSPer("SELECT descrizione FROM tacchi WHERE taccoID='"+taccoID+"'");
 	   	if(rs)
 	   	 {
 	     	if(mysql_num_rows(rs)==0) { taccoID="0"; tacco=""; }
 	     	else { row=mysql_fetch_row(rs); tacco=row[0]; }
 	     	mysql_free_result(rs);
 	     } 
 	    else { taccoID="0"; tacco=""; }
      rs=mainWin->sql->doQueryRSPer("SELECT descrizione FROM stagioni WHERE stagioneID='"+stagioneID+"'");
 	   	if(rs)
 	   	 {
 	   	  if(mysql_num_rows(rs)==0) { stagioneID="0"; stagione=""; }
 	   	  else { row=mysql_fetch_row(rs); stagione=row[0]; }
 	   	  mysql_free_result(rs);
 	   	 }
 	   	else { stagioneID="0"; stagione=""; }
 	   	QString giacenza=ARSTATO_GIACENZA;
 	 	  if(mainWin->sql->doQueryPer("UPDATE articoli SET codice='"+art+"', descrizione='"+descr+"', misura='"+misura+"', prezzo='"+prezzo+"', iva='"+iva+"', sconto='"+sconto+"', prezzo_pub='"+prezzo_pub+"', prezzo_lis='"+prezzo_pub+"', colore='"+colore+"', coloreID='"+coloreID+"', fondo='"+fondo+"', fondoID='"+fondoID+"', tacco='"+tacco+"', taccoID='"+taccoID+"', stagione='"+stagione+"', stagioneID='"+stagioneID+"' WHERE articoloID='"+articoloID+"' AND stato='"+giacenza+"' AND fatturaID='0' AND bollaID='0'"))
 	 	   {
 	 	   	int af;
 	 	   	
 	 	   	af=mainWin->sql->getAffected();
 	 	   	mainWin->sql->doQueryPer("UNLOCK TABLES");
        mainWin->sql->disconnetti();
        if(af==0) warnbox->setText(tr("L'articolo non  stato modificato poich\n in uso da un'altro utente,\noppure non  stato modificato nulla nell'articolo!"));
        else warnbox->setText(tr("Articolo modificato e salvato!"));
        warnbox->exec(); 
        mainWin->scaricoMerce();
       }
      else
       {
        mainWin->sql->doQueryPer("UNLOCK TABLES");
        mainWin->sql->disconnetti();
       }
     }
 	 }
 }

bool ARArticolo::setForm(QString articoloID, QString stato)
 {
  MYSQL_RES *rs;
  MYSQL_ROW row;
  QString tipo_articoloID,flColore,flFondo,flTacco,flStagione;

  if(stato==ARSTATO_VENDUTO || stato==ARSTATO_RESO) cornice->setEnabled(false);
  else                                              cornice->setEnabled(true);
  rs=sql->doQueryRS("SELECT * FROM articoli WHERE articoloID='"+articoloID+"'");
  if(rs)
   {
    if(mysql_num_rows(rs)!=1) { mysql_free_result(rs); return false; }
    row=mysql_fetch_row(rs);
    art->setFocus();
 	art->setText(row[6]);
 	descr->setText(row[8]);
 	misura->setText(row[22]);
 	prezzo->setText(row[17]);
 	iva->setText(row[20]);
 	sconto->setText(row[21]);
 	prezzo_pub->setText(row[18]);
 	tipo_articoloID=row[3];
 	settaInfo(QString(stato),QString(row[6]),QString(row[8]),QString(row[22]),articoloID,QString(row[1]),QString(row[32]),QString(row[26]),QString(row[27]));
    mysql_free_result(rs);
   }

  rs=sql->doQueryRS("SELECT colore,fondo,tacco,stagione FROM tipi_articolo WHERE tipo_articoloID='"+tipo_articoloID+"'");
  if(rs)
   {
    if(mysql_num_rows(rs)!=1) { mysql_free_result(rs); return false; }
    row=mysql_fetch_row(rs);
    flColore=row[0];
    flFondo=row[1];
    flTacco=row[2];
    flStagione=row[3];
 	if(flColore=="0")   listaColore->setEnabled(false);   else listaColore->setEnabled(true);
 	if(flFondo=="0")    listaFondo->setEnabled(false);    else listaFondo->setEnabled(true);
 	if(flTacco=="0")    listaTacco->setEnabled(false);    else listaTacco->setEnabled(true);
 	if(flStagione=="0") listaStagione->setEnabled(false); else listaStagione->setEnabled(true);
 	mysql_free_result(rs);
 	return true;
   }
  return false;
 }

void ARArticolo::clearForm()
 {
  art->setFocus();
  art->clear();
  descr->clear();
  misura->clear();
  prezzo->clear();
  iva->clear();
  prezzo_pub->clear();
 }

bool ARArticolo::checkForm()
 {
 	double d;
 	bool ok;
 	
 	if(quote(this->art->text())=="")
 	 {
 	  warnbox->setText(tr("Il campo articolo non pu essere vuoto"));
 	  warnbox->exec();
 	  return true;
 	 }
 	if(quote(this->descr->text())=="")
 	 {
 	  warnbox->setText(tr("Devi inserire la descrizione dell'articolo"));
 	  warnbox->exec();
 	  return true;
 	 }
 	if(quote(this->iva->text())=="")
 	 {
 	  warnbox->setText(tr("Devi inserire il valore dell'IVA"));
 	  warnbox->exec();
 	  return true;
 	 }
 	if(quote(this->prezzo->text())=="")
 	 {
 	  warnbox->setText(tr("Devi inserire il prezzo"));
 	  warnbox->exec();
 	  return true;
 	 }
 	if(quote(this->prezzo_pub->text())=="")
 	 {
 	  warnbox->setText(tr("Devi inserire il prezzo al pubblico"));
 	  warnbox->exec();
 	  return true;
 	 }
 	d=quote(this->prezzo->text()).toDouble(&ok);
 	if(!ok || d<0 || d>50000000)
 	 {
 	  warnbox->setText(tr("Prezzo errato, (da 0,00 a  50.000.000,00).\nCorreggere prezzo al pubblico!"));
    warnbox->exec();
 	 	return true;
 	 }
 	d=quote(this->prezzo_pub->text()).toDouble(&ok);
 	if(!ok || d<0 || d>50000000)
 	 {
 	  warnbox->setText(tr("Prezzo al pubblico errato, (da 0,00 a  50.000.000,00).\nCorreggere prezzo al pubblico!"));
    warnbox->exec();
 	 	return true;
 	 }
 	return false;
 }

void ARArticolo::settaInfo(QString stato,QString art,QString descr,QString misura,QString articoloID,QString fornitoreID,QString data,QString data_sr,QString ora_sr)
 {
  QString zeri,fornitore;
  QDate d   = QDate::fromString(data,"yyyy-MM-dd");
  QDate dsr = QDate::fromString(data_sr,"yyyy-MM-dd");
  QTime osr = QTime::fromString(ora_sr,"hh:mm:ss");
  MYSQL_RES *rs;
  MYSQL_ROW row;

  rs=sql->doQueryRS("SELECT nome FROM fornitori WHERE fornitoreID='"+fornitoreID+"'");
  if(rs)
   {
    if(mysql_num_rows(rs)!=1) fornitore="SCONOSCIUTO";
    else { row=mysql_fetch_row(rs); fornitore=row[0]; }
    mysql_free_result(rs);
   }
  for(int i=0;i<10-articoloID.length();i++) zeri+="0";
  this->info->setText("- Codice a barre articolo:<br>&nbsp;&nbsp;&nbsp;<b>"+zeri+articoloID+"</b><br><br>- Fornitore:<br>&nbsp;&nbsp;&nbsp;<b>"+fornitore+"</b><br><br>L'articolo risulta caricato in magazzino <b>"+d.toString("dddd, dd MMM yyyy")+"</b><br>");
  if(stato==ARSTATO_VENDUTO)  this->info->setText(this->info->text()+"<br>L'articolo risulta venduto <b>"+dsr.toString("dddd, dd MMM yyyy")+"</b>");
  if(stato==ARSTATO_RESO)     this->info->setText(this->info->text()+"<br>L'articolo risulta reso <b>"+dsr.toString("dddd, dd MMM yyyy")+"</b>");
  if(stato==ARSTATO_GIACENZA) this->info->setText(this->info->text()+"<br>L'articolo risulta giacente in magazzino (non ancora venduto)");
  if(stato==ARSTATO_GIACENZA)
   {
    this->cambia_data->setVisible(false);
    this->cambia->setVisible(false);
   }
  else
   {
    this->cambia_data->setVisible(true);
    this->cambia->setVisible(true);
    this->cambia->setDate(dsr);
    this->cambia->setTime(osr);
    if(stato==ARSTATO_RESO)    this->cambia_data->setText("Correggi data di reso");
    if(stato==ARSTATO_VENDUTO) this->cambia_data->setText("Correggi data di vendita");
   }
 }

void ARArticolo::populateListe()
 {
  QString coloreID,fondoID,taccoID,stagioneID;

  coloreID=prendiOptionID("coloreID",articoloID);
  populateOption(listaColore,"colori",coloreID);

  fondoID=prendiOptionID("fondoID",articoloID);
  populateOption(listaFondo,"fondi",fondoID);

  taccoID=prendiOptionID("taccoID",articoloID);
  populateOption(listaTacco,"tacchi",taccoID);

  stagioneID=prendiOptionID("stagioneID",articoloID);
  populateOption(listaStagione,"stagioni",stagioneID);

  // listaMisura ???
 }

bool ARArticolo::populateOption(QComboBox *lista, QString tb, QString valID)
 {
 	MYSQL_RES *rs;
 	MYSQL_ROW row;
 	int r=0;
 	QString ID;

 	lista->clear();
 	rs=sql->doQueryRS("SELECT * FROM "+tb+" ORDER BY descrizione");
 	lista->addItem("",QVariant("0"));
	if(rs)
 	 {
 	  while((row=mysql_fetch_row(rs)))
     {
     	// row[0]=ID row[2]=descrizione
     	ID=row[0];
     	lista->addItem(row[2],QVariant(row[0]));
     	r++;
     	if(ID==valID) lista->setCurrentIndex(r);
     }
    mysql_free_result(rs);
   }
  return r;
 }

QString ARArticolo::prendiOptionID(QString field, QString ID)
 {
 	MYSQL_RES *rs;
 	MYSQL_ROW row;
 	
 	rs=sql->doQueryRS("SELECT "+field+" FROM articoli WHERE articoloID="+ID);
 	if(rs)
 	 {
 	 	row=mysql_fetch_row(rs);
 	  ID=row[0];
 	  mysql_free_result(rs);
 	  return ID;
 	 }
 	return "0";
 }

QString ARArticolo::quote(QString str, bool like)
 {
 	if(like)
 	 {
 	  str.replace('\\',"\\\\\\\\");
 	  str.replace('\'',"\\'");
 	 }
 	else
   {
 	  str.replace('\\',"\\\\");
 	  str.replace('\'',"\\'");
   }
 	return str;
 }

void ARArticolo::setArticoloID(QString artID) { articoloID=artID; }

void ARArticolo::setAzione(int azio) { azione=azio; }

bool ARArticolo::isAzione(int azio) { if(azione==azio) return true; else return false; }
