#include <QPushButton>
#include <QGroupBox>
#include <QComboBox>
#include <QLabel>
#include <QHeaderView>
#include <QTableWidget>
#include <QTableWidgetItem>
#include <QMessageBox>
#include <QProgressBar>

#include "ARsql.h"
#include "ARFinestra.h"

#include "ARCarico.h"

ARCarico::ARCarico(QWidget *parent) : QWidget(parent)
 {
  mainWin=(ARFinestra *)parent;

  c_prima=6; // Numero colonne prima delle misure
  row_iva=111;    // Posizione nella tabella "tipi_articolo"
  row_sconto=112; // Posizione nella tabella "tipi_articolo"
  rows_campi=107; // Posizione iniziale nella tabella "tipi_articolo"
  iva="20";
  sconto="0";
  sql      = new ARsql(this);
  infobox  = new QMessageBox(tr("Informazioni"),"",QMessageBox::Question,0,0,0,this);
  yesnobox = new QMessageBox(tr("Scegli"),"",QMessageBox::Question,QMessageBox::Yes,QMessageBox::No,QMessageBox::NoButton,this);
  cornice  = new QGroupBox(tr("Carico merce in magazzino "),this);
  esegui   = new QPushButton(QIcon("pic/magazzino_add.png"),tr("Carica merce"),this);
  addrow   = new QPushButton(QIcon("pic/+.png"),tr("Ancora articoli"),this);
  updtot   = new QPushButton(QIcon("pic/upd.png"),tr("Aggiorna totali"),this);
  clsrow   = new QPushButton(QIcon("pic/cls.png"),tr("Pulisci riga"),this);

  _for      = new QLabel(tr("Fornitore"),this);
  _for->setGeometry(10,20,80,24);
  listaFor  = new QComboBox(this);
  listaFor->setGeometry(80,20,300,24);
  _tipo = new QLabel(tr("Tipo articolo"),this);
  _tipo->setGeometry(10,50,80,22);
  listaTipo = new QComboBox(this);
  listaTipo->setGeometry(80,50,250,22);
  _table = new QLabel(tr("Dati e quantit articoli"),this);
  _table->setGeometry(10,70,120,22);
  _totali = new QLabel(tr("Totali carico: "),this);
  progresso  = new QProgressBar(this);
  progresso->setVisible(false);

  num_mis=prendiEtichette(); // Da qui si ricava anche num_campi
  tot_row=12;
  listaColore=NULL; listaFondo=NULL; listaTacco=NULL; listaStagione=NULL;
  fl_misure = new QString[num_mis];
  fl_campi  = new QString[num_campi];
  table = new QTableWidget(this);
  table->verticalHeader()->hide();
  table->setAlternatingRowColors(true);
  table->setSelectionMode(QAbstractItemView::SingleSelection);
  table->setSelectionBehavior(QAbstractItemView::SelectRows);
 	for(int i=0; i<num_mis+c_prima+6;i++) { table->insertColumn(i); table->setColumnWidth(i,28); }
 	table->setHorizontalHeaderLabels(intestazione);
 	table->setColumnWidth(0,50);
 	table->setColumnWidth(1,90);
 	table->setColumnWidth(2,75);
 	table->setColumnWidth(3,70);
 	table->setColumnWidth(4,45);
 	table->setColumnWidth(5,80);
 	table->setColumnWidth(num_mis+c_prima+0,60); // Prezzo
 	table->setColumnWidth(num_mis+c_prima+1,26); // IVA
 	table->setColumnWidth(num_mis+c_prima+2,60); // Prezzo al pubblico
 	table->setColumnWidth(num_mis+c_prima+3,48); // Sconto
 	table->setColumnWidth(num_mis+c_prima+4,44); // Totale
 	table->setColumnWidth(num_mis+c_prima+5,40); // Pezzi
  clearCarico();
  
  connect(clsrow,SIGNAL(clicked()),this,SLOT(clearRiga()));
  connect(updtot,SIGNAL(clicked()),this,SLOT(updateTot()));
  connect(esegui,SIGNAL(clicked()),this,SLOT(caricaMerce()));
  connect(listaTipo,SIGNAL(activated(int)),this,SLOT(drawInputTipo(int)));
  connect(addrow,SIGNAL(clicked()),this,SLOT(addEmptyRow()));

  yesnobox->setWindowModality(Qt::WindowModal);
 	yesnobox->setButtonText(QMessageBox::Yes,tr("Si"));
 	yesnobox->setButtonText(QMessageBox::No,tr("No"));
  infobox->setWindowModality(Qt::WindowModal);
  //azione=ARFORNITORE_ADD;
  hide();
 }

ARCarico::~ARCarico() { delete[] fl_misure; delete[] fl_campi; delete[] listaColore; delete[] listaFondo; delete[] listaTacco; delete[] listaStagione; }

void ARCarico::settaGeometria(int lar, int alt)
 {
  setGeometry(0,61,lar,alt-111);
  table->setGeometry(10,90,lar-16,alt-230);
 	cornice->setGeometry(2,0,lar-2,alt-111);
 	clsrow->setGeometry(lar-314,50,100,24);
 	updtot->setGeometry(lar-210,50,100,24);
  addrow->setGeometry(lar-106,50,100,24);
  esegui->setGeometry(lar-99,alt-138,96,24);
  _totali->setGeometry(10,alt-138,250,22);
  progresso->setGeometry(480,alt-136,206,20);
 }

bool ARCarico::populateFor()
 {
 	MYSQL_RES *rs;
 	MYSQL_ROW row;
 	int r=0;
 	
 	listaFor->clear();
 	rs=sql->doQueryRS("SELECT fornitoreID,nome FROM fornitori ORDER BY nome");
	if(rs)
 	 {
 	  while((row=mysql_fetch_row(rs)))
     {
     	listaFor->addItem(row[1],QVariant(row[0]));
   	  r++;
     }
    mysql_free_result(rs);
   }
  return r;
 }

bool ARCarico::populateTipo()
 {
 	MYSQL_RES *rs;
 	MYSQL_ROW row;
 	int r=0;

 	listaTipo->clear();
 	rs=sql->doQueryRS("SELECT tipo_articoloID,descrizione FROM tipi_articolo ORDER BY descrizione");
	if(rs)
 	 {
 	  while((row=mysql_fetch_row(rs)))
     {
     	listaTipo->addItem(row[1],QVariant(row[0]));
   	  r++;
     }
    mysql_free_result(rs);
   }
  if(r!=0) drawInputTipo(0);
  return r;
 }

bool ARCarico::populateOptionAll(QComboBox *lista, int lun, QString tb)
 {
 	MYSQL_RES *rs;
 	MYSQL_ROW row;
 	int r=0;

 	lista->clear();
 	rs=sql->doQueryRS("SELECT * FROM "+tb+" ORDER BY descrizione");
 	for(int i=0;i<lun;i++) lista[i].addItem("",QVariant("0"));
	if(rs)
 	 {
 	  while((row=mysql_fetch_row(rs)))
     {
     	// row[0]=ID row[2]=descrizione
     	for(int i=0;i<lun;i++) lista[i].addItem(row[2],QVariant(row[0]));
   	  r++;
     }
    mysql_free_result(rs);
   }
  return r;
 }

bool ARCarico::populateOption(QComboBox *lista, QString tb)
 {
 	MYSQL_RES *rs;
 	MYSQL_ROW row;
 	int r=0;

 	lista->clear();
 	rs=sql->doQueryRS("SELECT * FROM "+tb+" ORDER BY descrizione");
 	lista->addItem("",QVariant("0"));
	if(rs)
 	 {
 	  while((row=mysql_fetch_row(rs)))
     {
     	// row[0]=ID row[2]=descrizione
     	lista->addItem(row[2],QVariant(row[0]));
   	  r++;
     }
    mysql_free_result(rs);
   }
  return r;
 }

void ARCarico::updateTot() { if(checkCarico()) calcTotali(); }

void ARCarico::caricaMerce()
 {
 	QString fornitoreID,tipo_articoloID,query;
 	QString codice,codice_extra,descrizione,prezzo,prezzo_pub,prezzo_lis,sconto,iva,misura,data,ora,ts,stato,fatturato,bolla;
 	QString coloreID,colore,fondoID,fondo,taccoID,tacco,stagioneID,stagione;
 	QComboBox *lColore,*lFondo,*lTacco,*lStagione;
 	int totp;
 	bool error;
  
  if(checkCarico())
   {
   	totp=calcTotali();
  	yesnobox->setText(tr("Confermi carico articoli in magazzino?"));
    if(yesnobox->exec()==QMessageBox::Yes)
     {
 	 	 	MYSQL_RES *rs;
 	    MYSQL_ROW row;

     	progresso->setVisible(true); progresso->setValue(0);
     	int pezzi=0;
   	  error=false;
 	 	  fornitoreID=getFornitoreID();
 	    tipo_articoloID=getTipoArticoloID();

	   	sql->connetti();
 	   	sql->doQueryPer("LOCK TABLES articoli WRITE, colori WRITE, fondi WRITE, tacchi WRITE, stagioni WRITE");
      for(int r=0;r<getTotRows();r++)
       {
     	  codice=quote(getElemento(r,0));
     	  descrizione=quote(getElemento(r,1));
     	  prezzo=quote(getElemento(r,getTotColumns()-5,"0"));
     	  iva=quote(getElemento(r,getTotColumns()-4,"0"));
     	  prezzo_pub=quote(getElemento(r,getTotColumns()-3,"0"));
     	  sconto=quote(getElemento(r,getTotColumns()-2,"0"));
     	  // Colore
     	  if(!table->isColumnHidden(2))
     	   {
     	    lColore=(QComboBox *)table->cellWidget(r,2);
     	    coloreID=lColore->itemData(lColore->currentIndex()).toString();
     	    colore=lColore->itemText(lColore->currentIndex());
     	   	rs=sql->doQueryRSPer("SELECT descrizione FROM colori WHERE coloreID='"+coloreID+"'");
    	   	if(rs)
 	      	 {
 	   	      if(mysql_num_rows(rs)==0) { coloreID="0"; colore=""; }
 	   	      else { row=mysql_fetch_row(rs); colore=row[0]; }
 	   	      mysql_free_result(rs);
 	   	     }
 	   	    else { coloreID="0"; colore=""; }   
     	   }
     	  else { coloreID="0"; colore=""; }
     	  // Fondo
     	  if(!table->isColumnHidden(3))
     	   {
     	    lFondo=(QComboBox *)table->cellWidget(r,3);
     	    fondoID=lFondo->itemData(lFondo->currentIndex()).toString();
     	    fondo=lFondo->itemText(lFondo->currentIndex());
    	   	rs=sql->doQueryRSPer("SELECT descrizione FROM fondi WHERE fondoID='"+fondoID+"'");
 	   	    if(rs)
 	   	     {
   	   	    if(mysql_num_rows(rs)==0) { fondoID="0"; fondo=""; }
 	    	    else { row=mysql_fetch_row(rs); fondo=row[0]; }
 	    	    mysql_free_result(rs);
 	         }
 	        else { fondoID="0"; fondo=""; }
     	   }
     	  else { fondoID="0"; fondo=""; }
     	  // Tacco
     	  if(!table->isColumnHidden(4))
     	   {     	  
     	    lTacco=(QComboBox *)table->cellWidget(r,4);
     	    taccoID=lTacco->itemData(lTacco->currentIndex()).toString();
     	    tacco=lTacco->itemText(lTacco->currentIndex());
          rs=sql->doQueryRSPer("SELECT descrizione FROM tacchi WHERE taccoID='"+taccoID+"'");
 	   	    if(rs)
 	   	     {
 	     	    if(mysql_num_rows(rs)==0) { taccoID="0"; tacco=""; }
 	     	    else { row=mysql_fetch_row(rs); tacco=row[0]; }
 	     	    mysql_free_result(rs);
 	         } 
 	        else { taccoID="0"; tacco=""; } 
     	   }
     	  else { taccoID="0"; tacco=""; }
     	  // Stagione
     	  if(!table->isColumnHidden(5))
     	   {     	  
     	    lStagione=(QComboBox *)table->cellWidget(r,5);
     	    stagioneID=lStagione->itemData(lStagione->currentIndex()).toString();
     	    stagione=lStagione->itemText(lStagione->currentIndex());
          rs=sql->doQueryRSPer("SELECT descrizione FROM stagioni WHERE stagioneID='"+stagioneID+"'");
 	   	    if(rs)
 	   	     {
 	   	      if(mysql_num_rows(rs)==0) { stagioneID="0"; stagione=""; }
 	   	      else { row=mysql_fetch_row(rs); stagione=row[0]; }
 	   	      mysql_free_result(rs);
 	   	     }
 	   	    else { stagioneID="0"; stagione=""; }
         }
        else { stagioneID="0"; stagione=""; }
 	      ts=sql->timeStamp(); data=sql->data(ts); ora=sql->ora(ts);
 	      for(int c=c_prima;c<getTotColumns()-5;c++)
 	       {
 	       	if(!table->isColumnHidden(c))
 	       	 {
 	     	    QString qta;
 	     	    qta=getElemento(r,c);
 	          for(int i=0;i<qta.toInt();i++)
 	           {
 	            if(etichette.at(c-c_prima)=="Q.t") misura="-"; else misura=quote(etichette.at(c-c_prima));
 	       	    stato=ARSTATO_GIACENZA;
 	       	    fatturato="0";
 	       	    bolla="0";
 	       	    prezzo_lis=prezzo_pub;
 	            sql->doQueryPer("INSERT INTO articoli VALUES('0','"+fornitoreID+"','0','"+tipo_articoloID+"','0','0','"+codice+"','"+codice_extra+"','"+descrizione+"','"+coloreID+"','"+colore+"','"+fondoID+"','"+fondo+"','"+taccoID+"','"+tacco+"','"+stagioneID+"','"+stagione+"','"+prezzo+"','"+prezzo_pub+"',"+prezzo_lis+",'"+iva+"','"+sconto+"','"+misura+"','"+stato+"','"+fatturato+"','"+bolla+"','0000-00-00','00:00:00','0','0000-00-00','00:00:00','0','"+data+"','"+ora+"','"+ts+"')");
 	            pezzi++;
 	            progresso->setValue((int)(((float)pezzi/(float)totp)*100));
             }
           }
 	       }
     	 }
     
     	sql->doQueryPer("UNLOCK TABLES");
      sql->disconnetti(); 
     	 
     	QString spezzi;
     	spezzi.setNum(pezzi);
     	infobox->setText(spezzi+tr(" articoli inseriti in magazzino!"));
   	  infobox->exec();
   	  clearCarico();
   	  progresso->setVisible(false);
     }
 	 }
 }


void ARCarico::clearRiga() { if(table->currentRow()>=0) clsRow(table->currentRow()); }

void ARCarico::clsRow(int r)
 {
  for(int c=0;c<=getTotColumns();c++)
   {
 	  if(table->item(r,c))
 	   {
 	   	if(c==num_mis+c_prima+1)
 	   	 {
 	   	  table->item(r,num_mis+c_prima+1)->setText(iva);
 	   	 }
 	   	if(c==num_mis+c_prima+3)
 	   	 {
 	   	  table->item(r,num_mis+c_prima+3)->setText(sconto);
 	   	 }
 	   	if(c!=num_mis+c_prima+1 && c!=num_mis+c_prima+3)
 	   	 {
 	      table->item(r,c)->setText("");
 	      table->setItem(r,c,NULL);
 	     }
 	   }
   }
  QComboBox *aux_cb;
  aux_cb=(QComboBox *)table->cellWidget(r,2); aux_cb->setCurrentIndex(0);
  aux_cb=(QComboBox *)table->cellWidget(r,3); aux_cb->setCurrentIndex(0);
  aux_cb=(QComboBox *)table->cellWidget(r,4); aux_cb->setCurrentIndex(0);
 	aux_cb=(QComboBox *)table->cellWidget(r,5); aux_cb->setCurrentIndex(0);
 }

int ARCarico::checkElemento(int r, int c, int err, bool ckint, bool ckdouble)
 {
 	bool ok;
 	int n;
 	double d;
 	
 	if(!table->item(r,c)) return err;
 	if(table->item(r,c)->text()=="" && (ckint || ckdouble)) table->item(r,c)->setText("0");
 	if(table->item(r,c)->text()=="") return err;
 	if(ckint)
 	 {
 	 	n=table->item(r,c)->text().toInt(&ok);
 	 	if(!ok) return err;
 	 	if(n<0) return err;
 	 	if(n>100) return err+100;
 	 }
 	if(ckdouble)
 	 {
 	 	d=table->item(r,c)->text().toDouble(&ok);
 	 	if(!ok) return err;
 	 	if(d<0) return err;
 	 	if(d>50000000) return err+100;
 	 }
 	if(table->item(r,c)->text().length()>255) return err+100;
 	return 0;
 }

int ARCarico::checkRiga(int r)
 {
 	bool any=false;
 	int error=0;

 	for(int c=c_prima;c<getTotColumns()-4;c++) { if(table->item(r,c) && !table->isColumnHidden(c)) any=true; }
  if(any)
   {
    error=checkElemento(r,0,1); if(error!=0) return error; // Articolo
    error=checkElemento(r,1,2); if(error!=0) return error; // Descrizione
    for(int c=c_prima;c<getTotColumns()-5;c++)
     {
   	  if(table->item(r,c) && !table->isColumnHidden(c)) error=checkElemento(r,c,3,true); if(error!=0) return error; // quantit (solo colonne attive)
     }
    error=checkElemento(r,getTotColumns()-5,4,false,true); if(error!=0) return error; // Prezzo
    error=checkElemento(r,getTotColumns()-4,5,true);       if(error!=0) return error; // IVA
    error=checkElemento(r,getTotColumns()-3,6,false,true); if(error!=0) return error; // Prezzo pub
    error=checkElemento(r,getTotColumns()-2,7,false,true); if(error!=0) return error; // Sconto
 	 }
 	return 0;
 }

bool ARCarico::checkCarico()
 {
 	int error=0, rigaerrata=0;
 	
 	for(int r=0;r<getTotRows();r++) { error=checkRiga(r); rigaerrata=r; if(error!=0) break; }
 	if(error==1) infobox->setText(tr("Codice articolo vuoto, inserire il codice!"));
 	if(error==2) infobox->setText(tr("Descrizione articolo vuoto, inserire la descrizione!"));
 	if(error==3) infobox->setText(tr("Quantit errata, correggere quantit!"));
 	if(error==4) infobox->setText(tr("Formato prezzo errato, correggere prezzo!"));
 	if(error==5) infobox->setText(tr("Formato iva errato, correggere iva!"));
 	if(error==6) infobox->setText(tr("Formato prezzo al pubblico errato, correggere prezzo al pubblico!"));
 	if(error==7) infobox->setText(tr("Percentuale di sconto errata, correggere sconto!"));
 	if(error==101) infobox->setText(tr("Codice articolo troppo lungo, massimo 255 caratteri!"));
 	if(error==102) infobox->setText(tr("Descrizione troppo lunga, massimo 255 caratteri!"));
  if(error==103) infobox->setText(tr("Quantit troppo alta, massimo 100.\nCorreggere quantit!"));
  if(error==104) infobox->setText(tr("Prezzo troppo alto, massimo 50.000.000,00.\nCorreggere prezzo!"));
  if(error==105) infobox->setText(tr("L'IVA pu essere massimo 100.\nCorreggere IVA!"));
 	if(error==106) infobox->setText(tr("Prezzo al pubblico troppo alto, massimo 50.000.000,00.\nCorreggere prezzo al pubblico!"));
 	if(error==107) infobox->setText(tr("La percentuale di sconto pu essere massimo 100.\nCorreggere sconto!"));
 	table->selectRow(rigaerrata);
 	if(!error) { table->clearSelection(); return true; } else { infobox->exec(); return false; }
 }

int ARCarico::calcTotali()
 {
  QString stot,spz,stotali,aux1,aux2;
  double totale=0,tot=0;
  int pezzi=0,pz=0;
 	
  for(int r=0;r<getTotRows();r++)
   {
  	pz=0; tot=0;
  	if(table->item(r,getTotColumns()-5)) tot=table->item(r,getTotColumns()-5)->text().toDouble();
  	for(int c=2;c<getTotColumns()-5;c++)
  	 {
      if(!table->isColumnHidden(c) && table->item(r,c)) pz+=table->item(r,c)->text().toInt();
  	 }
 	tot=pz*tot;
 	totale=totale+tot; pezzi=pezzi+pz;
 	spz.sprintf("%d",pz);
 	stot.sprintf("%.2lf",tot);
 	table->setItem(r,getTotColumns()-1,inserisciRiga(stot,Qt::AlignRight)); // tot prezzo
 	table->setItem(r,getTotColumns(),inserisciRiga(spz,Qt::AlignRight));    // tot pezzi
   }
  aux1.sprintf("Totali carico: Imponibile %.2lf ",totale);
  aux2.sprintf(" (%d pezzi)",pezzi);
  stotali = aux1+mainWin->CURRENCY_NAME+aux2;
  _totali->setText(stotali);
  return pezzi;
 }

void ARCarico::clearCarico()
 {
  tot_row=12;
  if(listaColore) { delete[] listaColore; }
  listaColore = new QComboBox[tot_row]; populateOptionAll(listaColore,tot_row,"colori");
  if(listaFondo) { delete[] listaFondo; }
  listaFondo = new QComboBox[tot_row]; populateOptionAll(listaFondo,tot_row,"fondi");
  if(listaTacco) { delete[] listaTacco; }
  listaTacco = new QComboBox[tot_row]; populateOptionAll(listaTacco,tot_row,"tacchi");
  if(listaStagione) { delete[] listaStagione; }
  listaStagione = new QComboBox[tot_row]; populateOptionAll(listaStagione,tot_row,"stagioni");
  
  table->setRowCount(tot_row);
  for(int i=0;i<tot_row;i++)
 	 {
 	  table->setRowHeight(i,22);
 	  for(int j=0;j<num_mis+c_prima+1+5;j++) table->setItem(i,j,NULL);
 	 	table->setItem(i,num_mis+c_prima+1,inserisciRiga(iva));
 	 	table->setItem(i,num_mis+c_prima+3,inserisciRiga(sconto));
 	 	table->setCellWidget(i,2,&listaColore[i]);
 	 	table->setCellWidget(i,3,&listaFondo[i]);
 	 	table->setCellWidget(i,4,&listaTacco[i]);
 	 	table->setCellWidget(i,5,&listaStagione[i]);
 	 }
 	_totali->setText(tr("Totali carico: "));
 	table->clearSelection();
 }

void ARCarico::drawInputTipo(int indice)
 {
 	QString tipo_articoloID;
 	MYSQL_RES *rs;
 	MYSQL_ROW row;

 	tipo_articoloID=listaTipo->itemData(indice).toString();
 	rs=sql->doQueryRS("SELECT * FROM tipi_articolo WHERE tipo_articoloID='"+tipo_articoloID+"'");
 	if(mysql_num_rows(rs)==1)
 	 {
 	  row=mysql_fetch_row(rs);
 	  for(int i=0;i<num_mis;i++) fl_misure[i]=row[i+2];
 	  iva=row[row_iva];
 	  sconto=row[row_sconto];
 	  for(int i=0;i<tot_row;i++) table->setItem(i,num_mis+c_prima+1,inserisciRiga(iva));
 	  for(int i=0;i<tot_row;i++) table->setItem(i,num_mis+c_prima+3,inserisciRiga(sconto));
    for(int i=rows_campi;i<rows_campi+num_campi;i++) { fl_campi[i-rows_campi]=row[i]; }
 	  mysql_free_result(rs);
 	 }
  else
   {
    infobox->setText(tr("Tipologia articoli inesistente!"));
	  infobox->exec();
	  for(int i=0;i<num_mis;i++) table->hideColumn(i+c_prima);
	  mysql_free_result(rs);
    return;
   }
  for(int i=0;i<num_mis;i++)
   {
   	if(fl_misure[i]=="0") { table->hideColumn(i+c_prima); } else { table->showColumn(i+c_prima); }
   }
  for(int i=rows_campi;i<rows_campi+num_campi;i++)
   { 
   	if(fl_campi[i-rows_campi]=="0") { table->hideColumn(2+i-rows_campi); } else { table->showColumn(2+i-rows_campi); }
   }
 }

void ARCarico::addEmptyRow()
 {
 	QComboBox *aux_colori   = new QComboBox();
 	QComboBox *aux_fondi    = new QComboBox();
 	QComboBox *aux_tacchi   = new QComboBox();
 	QComboBox *aux_stagioni = new QComboBox();

 	table->setRowCount(tot_row+1);
 	table->setRowHeight(tot_row,22);
 	table->setItem(tot_row,num_mis+c_prima+1,inserisciRiga(iva));
 	table->setItem(tot_row,num_mis+c_prima+3,inserisciRiga(sconto));
 	populateOption(aux_colori,"colori"); table->setCellWidget(tot_row,2,aux_colori);
 	populateOption(aux_fondi,"fondi"); table->setCellWidget(tot_row,3,aux_fondi);
 	populateOption(aux_tacchi,"tacchi"); table->setCellWidget(tot_row,4,aux_tacchi);
 	populateOption(aux_stagioni,"stagioni"); table->setCellWidget(tot_row,5,aux_stagioni);
 	tot_row++;
 }

QTableWidgetItem *ARCarico::inserisciRiga(const char *str, int alig)
 {
 	QTableWidgetItem *item;
 	
 	item = new QTableWidgetItem();
 	item->setText(str);
 	item->setTextAlignment(alig);
 	return item;
 }

QTableWidgetItem *ARCarico::inserisciRiga(QString str, int alig)
 {
 	QTableWidgetItem *item;
 	
 	item = new QTableWidgetItem();
 	item->setText(str);
 	item->setTextAlignment(alig);
 	return item;
 }

QString ARCarico::getFornitoreID()
 {
 	return listaFor->itemData(listaFor->currentIndex()).toString();
 }

QString ARCarico::getTipoArticoloID()
 {
 	return listaTipo->itemData(listaTipo->currentIndex()).toString();
 }

QString ARCarico::getElemento(int r, int c, QString def)
 {
 	if(table->item(r,c)) return table->item(r,c)->text(); else return def;
 }

int ARCarico::prendiEtichette()
 {
 	etichette << "1" << "1" << "2" << "2" << "3" << "3" << "4" << "4" << "5" << "5" << "6" << "6" << "7" << "7" << "8" << "8" << "9" << "9" << "10" << "10" << "11" << "11" << "12" << "12" << "13" << "13" << "14" << "14" << "15" << "15" << "16" << "16" << "17" << "17" << "18" << "18" << "19" << "19" << "20" << "20" << "21" << "21" << "22" << "22" << "23" << "23" << "24" << "24" << "25" << "25" << "26" << "26" << "27" << "27" << "28" << "28" << "29" << "29" << "30" << "30" << "31" << "31" << "32" << "33" << "34" << "34" << "35" << "35" << "36" << "36" << "37" << "37" << "38" << "38" << "39" << "39" << "40" << "40" << "41" << "41" << "42" << "42" << "43" << "43" << "44" << "44" << "45" << "45" << "46" << "46" << "47" << "47" << "48" << "49" << "50" << "XXXS" << "XXS" << "XS" << "S" << "M" << "L" << "XL" << "XXL" << "XXXL" << "Q.t";
 	intestazione << "Art." << "Descrizione" << "Colore" << "Fondo" << "Tacco" << "Stagione" << etichette << "Prezzo" << "IVA" << "Pr. pub." << "Sconto" << "Totale" << "Pezzi";
 	etich_campi << "Colore" << "Fondo" << "Tacco" << "Stagione";
 	num_campi=etich_campi.count();
 	return etichette.count();
 }

QString ARCarico::quote(QString str, bool like)
 {
 	if(like)
 	 {
 	  str.replace('\\',"\\\\\\\\");
 	  str.replace('\'',"\\'");
 	 }
 	else
   {
 	  str.replace('\\',"\\\\");
 	  str.replace('\'',"\\'");
   }
 	return str;
 }

QStringList ARCarico::getEtichette() { return etichette; }

QStringList ARCarico::getEtichCampi() { return etich_campi; }

int ARCarico::getRowIva() { return row_iva; }

int ARCarico::getRowStartCampi() { return rows_campi; }

int ARCarico::getTotRows() { return tot_row; }

int ARCarico::getTotColumns() { return num_mis+c_prima+5; }

/*
void ARCarico::setAzione(int azio) { azione=azio; }

bool ARCarico::isAzione(int azio) { if(azione==azio) return true; else return false; }
*/
