#include <QPushButton>
#include <QGroupBox>
#include <QComboBox>
#include <QLabel>
#include <QDateEdit>
#include <QDate>
#include <QLineEdit>
#include <QTextEdit>
#include <QMessageBox>

#include "ARsql.h"
#include "ARFinestra.h"

#include "ARScadenza.h"

ARScadenza::ARScadenza(QWidget *parent) : QWidget(parent)
 {
  QString aux;

  mainWin=(ARFinestra *)parent;
  sql = new ARsql(this);
  infobox = new QMessageBox(tr("Informazioni"),"",QMessageBox::Question,0,0,0,this);
  warnbox = new QMessageBox(tr("Avviso"),"",QMessageBox::Warning,0,0,0,this);
  cornice = new QGroupBox(tr("Inserisci nuova scadenza "),this);
  cornice->setGeometry(2,0,650,420);

  QDate cd = QDate::currentDate();
  _for     = new QLabel(tr("Fornitore"),this);    _for->setGeometry      (10,  20, 80, 24);
  listaFor = new QComboBox(this);                 listaFor->setGeometry  (90,  20,540, 24);
  _descr   = new QLabel(tr("Descrizione"),this);  _descr->setGeometry    (10,  50, 80, 24);
  descr    = new QLineEdit("",this);              descr->setGeometry     (90,  50,540, 24);
  aux = "Somma ("+mainWin->CURRENCY_NAME+")";
  _somma   = new QLabel(tr(aux.toAscii()),this);  _somma->setGeometry    (10,  80, 80, 24);
  somma    = new QLineEdit("",this);              somma->setGeometry     (90,  80,140, 24);
  _scade   = new QLabel(tr("Scadenza il:"),this); _scade->setGeometry    (240, 80, 80, 24);
  scade    = new QDateEdit(cd,this);              scade->setGeometry     (310, 80,130, 24);
  _stato   = new QLabel(tr("Stato"),this);        _stato->setGeometry    (450, 80, 80, 24);
  listaStato = new QComboBox(this);               listaStato->setGeometry(490, 80,140, 24);
  listaStato->addItem(tr("Da pagare"),QVariant(0));
  listaStato->addItem(tr("Pagato"),QVariant(1));
  _note    = new QLabel(tr("<b>Note:</b>"),this); _note->setGeometry     (  6,105, 80, 24);
  note     = new QTextEdit("",this);              note->setGeometry      ( 10,130,620,120);
  
  descr->setMaxLength(255);
  somma->setValidator(new QDoubleValidator(-50000000.0,50000000.0,2,somma));
  scade->setDisplayFormat("dd-MMMM-yyyy");

 	esegui = new QPushButton(QIcon("pic/scadenze_add.png"),tr("Inserisci"),this);
 	esegui->setGeometry(543,393,106,24);

  connect(esegui,SIGNAL(clicked()),this,SLOT(addScadenza()));

 	infobox->setWindowModality(Qt::WindowModal);
 	warnbox->setWindowModality(Qt::WindowModal);
  setGeometry(0,61,680,440);
  azione=ARSCADENZA_ADD;  
  hide();
 }

void ARScadenza::addScadenza()
 {
 	QString fornitoreID=listaFor->itemData(listaFor->currentIndex()).toString();
  QString descr=quote(this->descr->text());
  QString somma=quote(this->somma->text()); if(somma=="") somma="0";
  QString data_scadenza=scade->date().toString("yyyy-MM-dd");
  QString note=quote(this->note->toPlainText());
  QString stato=listaStato->itemData(listaStato->currentIndex()).toString();
  QString ts=sql->timeStamp();
  QString data=sql->data(ts);
  QString ora=sql->ora(ts);

 	if(isAzione(ARSCADENZA_ADD))
 	 {
 	  if(!checkForm())
 	   {
 	   	if(sql->doQuery("INSERT INTO scadenze VALUES('0','"+fornitoreID+"','"+descr+"','"+note+"','"+stato+"','"+somma+"','"+data_scadenza+"','00:00:00','0','"+data+"','"+ora+"','"+ts+"')"))
       {
        infobox->setText(tr("Nuova scadenza inserita"));
        infobox->exec();
        clearForm();
 	     }
 	   }
 	 }
 	if(isAzione(ARSCADENZA_EDIT))
 	 {
 	  if(!checkForm())
 	   {
 	 	  if(sql->doQuery("UPDATE scadenze SET fornitoreID='"+fornitoreID+"', descrizione='"+descr+"', somma='"+somma+"', data_scadenza='"+data_scadenza+"', stato='"+stato+"', note='"+note+"' WHERE scadenzaID='"+scadenzaID+"'"))
 	 	   {
        warnbox->setText(tr("Scadenza modificata e salvata!"));
        warnbox->exec(); 
        mainWin->visScadenze();
       }
     }
 	 }
 }

bool ARScadenza::setForm(QString scadenzaID)
 {
 	MYSQL_RES *rs;
 	MYSQL_ROW row;

  rs=sql->doQueryRS("SELECT * FROM scadenze WHERE scadenzaID='"+scadenzaID+"'");
  if(rs)
   {
    if(mysql_num_rows(rs)!=1) { mysql_free_result(rs); return false; }
    row=mysql_fetch_row(rs);
    populateFor(row[1]);
    descr->setFocus();
    descr->setText(row[2]);
 	  somma->setText(row[5]);
 	  scade->setDate(QDate::fromString(row[6],"yyyy-MM-dd"));
 	  listaStato->setCurrentIndex(QString(row[4]).toInt());
 	  note->setPlainText(row[3]);
 	  mysql_free_result(rs);
 	  return true;
 	 }
  return false;
 }

void ARScadenza::clearForm()
 {
 	populateFor("0");
 	listaStato->setCurrentIndex(0);
 	descr->setFocus();
 	descr->clear();
 	somma->clear();
 	scade->setDate(QDate::currentDate());
 	note->clear();
 }

bool ARScadenza::checkForm()
 {
 	if(quote(this->descr->text())=="")
 	 {
 	  warnbox->setText(tr("Devi inserire la descrizione"));
 	  warnbox->exec();
 	  descr->setFocus();
 	  return true;
 	 }
 	if(quote(this->somma->text())=="")
 	 {
 	  warnbox->setText(tr("Devi inserire la somma della scadenza"));
 	  warnbox->exec();
 	  somma->setFocus();
 	  return true;
 	 }
 	if(quote(this->note->toPlainText()).length()>1024)
 	 {
 	  warnbox->setText(tr("Campo note troppo lungo,\nmassimo 1000 caratteri."));
 	  warnbox->exec();
 	  return true;
 	 }
 	return false;
 }

bool ARScadenza::populateFor(QString val)
 {
 	MYSQL_RES *rs;
 	MYSQL_ROW row;
 	int r=0,v=0;
 	QString aux;
 	
 	listaFor->clear();
 	listaFor->addItem(tr("SCADENZA NON LEGATA AD UN FORNITORE"),QVariant(0));
 	rs=sql->doQueryRS("SELECT fornitoreID,nome FROM fornitori ORDER BY nome");
	if(rs)
 	 {
 	  while((row=mysql_fetch_row(rs)))
     {
     	listaFor->addItem(row[1],QVariant(row[0]));
   	  r++;
     	aux=row[0]; if(aux==val) v=r;
     }
    mysql_free_result(rs);
   }
  listaFor->setCurrentIndex(v);
  return r;
 }

QString ARScadenza::quote(QString str, bool like)
 {
 	if(like)
 	 {
 	  str.replace('\\',"\\\\\\\\");
 	  str.replace('\'',"\\'");
 	 }
 	else
   {
 	  str.replace('\\',"\\\\");
 	  str.replace('\'',"\\'");
   }
 	return str;
 }

void ARScadenza::settaFormAdd()
 {
 	esegui->setText(tr("Inserisci"));
 	esegui->setIcon(QIcon("pic/scadenze_add.png"));
 	cornice->setTitle(tr("Inserisci nuova scadenza "));
  clearForm();
 }

void ARScadenza::settaFormEdit()
 {
 	esegui->setText(tr("Salva modifiche"));
 	esegui->setIcon(QIcon("pic/floppy.png"));
	cornice->setTitle(tr("Visualizza scadenza "));
 }

void ARScadenza::setScadenzaID(QString scaID) { scadenzaID=scaID; }

void ARScadenza::setAzione(int azio) { azione=azio; }

bool ARScadenza::isAzione(int azio) { if(azione==azio) return true; else return false; }
