#include <QTextDocument>
#include <QMessageBox>
#include <QPrinter>
#include <QPushButton>
#include <QLineEdit>
#include <QDateEdit>
#include <QComboBox>
#include <QPrintDialog>
#include <QPainter>
#include <QTableWidget>
#include <QDate>
#include <QSpinBox>

#include "ARFinestra.h"
#include "ARScarico.h"
#include "ARBarraInfo.h"
#include "ARsql.h"
#include "ARDialogAnteprima.h"
#include "ARDialogStampe.h"
#include "ARAzienda.h"

#include "ARStampante.h"

ARStampante::ARStampante(QWidget *parent) : QWidget(parent)
 {
  mainWin=(ARFinestra *)parent;
  tipo=ARSTAMPANTE_ETICHETTE;
  sql       = new ARsql(this);
  infobox   = new QMessageBox(tr("Informazioni"),"",QMessageBox::Question,0,0,0,parent);
  warnbox   = new QMessageBox(tr("Avviso"),"",QMessageBox::Warning,0,0,0,this);
  stampante = new QPrinter();
  stampabox = new QPrintDialog(stampante,this);
  prestampabox = new ARDialogStampe(this);
  prebox    = new ARDialogAnteprima(this);

  connect(prebox->setstampa,SIGNAL(clicked()),this,SLOT(settaStampante()));
  connect(prebox->prefstampa,SIGNAL(clicked()),this,SLOT(settaStampe()));
  connect(prebox->prima,SIGNAL(clicked()),this,SLOT(primaPagina()));
  connect(prebox->succ,SIGNAL(clicked()),this,SLOT(succPagina()));
  connect(prebox->prec,SIGNAL(clicked()),this,SLOT(precPagina()));
  connect(prebox->ini_r,SIGNAL(valueChanged(int)),this,SLOT(cambiaRC(int)));
  connect(prebox->ini_c,SIGNAL(valueChanged(int)),this,SLOT(cambiaRC(int)));

  setGeometry(0,0,0,0);
  infobox->setWindowModality(Qt::WindowModal);
  warnbox->setWindowModality(Qt::WindowModal);
  //stampabox->setButtonText(QMessageBox::Yes,tr("Accetta")); // ???
  stampabox->setWindowTitle(tr("Impostazioni di stampa")); // ???
  initBcodeLUT();
 }

void ARStampante::cambiaRC(int i)
 {
 	if(aux_table)
 	 {
 	  if(tipo==ARSTAMPANTE_ETICHETTE)        stampaRealEtichette(aux_table,true,1);
 	  if(tipo==ARSTAMPANTE_INTESTAZIONI_CLI) stampaRealIntestCli(aux_table,true,1);
 	  if(tipo==ARSTAMPANTE_INTESTAZIONI_FOR) stampaRealIntestFor(aux_table,true,1);
 	 }
 	if(tipo==ARSTAMPANTE_INTESTAZIONI_AZI) stampaRealIntestAzi(aux_table,true,1);
 	prebox->setPagina(1);
 	prebox->ridisegna();
 }

void ARStampante::prendiPref(int cur)
 {
 	if(cur==0) // Etichette
 	 {
    mdestro=sql->getPref(1,"pref_stampante_et");
    malto=sql->getPref(2,"pref_stampante_et");
    roriz=sql->getPref(3,"pref_stampante_et");
    msinistro=sql->getPref(4,"pref_stampante_et");
    mbasso=sql->getPref(5,"pref_stampante_et");
    rvert=sql->getPref(6,"pref_stampante_et");
    num_r=sql->getPref(7,"pref_stampante_et");
    num_c=sql->getPref(8,"pref_stampante_et");
    barcodeon=sql->getPref(9,"pref_stampante_et");
    bcodesmall=sql->getPref(10,"pref_stampante_et");
    bcodealt=sql->getPref(11,"pref_stampante_et").toUInt();
    campi_vis=sql->getPref(12,"pref_stampante_et").toUInt();
   }
 	if(cur==1) // Intestazioni
 	 {
    mdestro=sql->getPref(1,"pref_stampante_in");
    malto=sql->getPref(2,"pref_stampante_in");
    roriz=sql->getPref(3,"pref_stampante_in");
    msinistro=sql->getPref(4,"pref_stampante_in");
    mbasso=sql->getPref(5,"pref_stampante_in");
    rvert=sql->getPref(6,"pref_stampante_in");
    num_r=sql->getPref(7,"pref_stampante_in");
    num_c=sql->getPref(8,"pref_stampante_in");
    titcli=sql->getPref(9,"pref_stampante_in");
    titfor=sql->getPref(10,"pref_stampante_in");
   }
 }  

void ARStampante::settaStampante()
 {
 	if(stampabox->exec()==QDialog::Accepted)
   {
   	if(tipo==ARSTAMPANTE_ETICHETTE) stampaRealEtichette(aux_table,true,prebox->getPagina());
   	if(tipo==ARSTAMPANTE_INTESTAZIONI_FOR) stampaRealIntestFor(aux_table,true,prebox->getPagina());
   	if(tipo==ARSTAMPANTE_INTESTAZIONI_CLI) stampaRealIntestCli(aux_table,true,prebox->getPagina());
    if(tipo==ARSTAMPANTE_INTESTAZIONI_AZI) stampaRealIntestAzi(aux_table,true,prebox->getPagina());
   	prebox->ridisegna();
   }
 }

void ARStampante::settaStampe()
 {
 	prestampabox->loadPrefStampa();
  if(tipo==ARSTAMPANTE_ETICHETTE) prestampabox->attivaTab(0);
  if(tipo==ARSTAMPANTE_INTESTAZIONI_FOR || tipo==ARSTAMPANTE_INTESTAZIONI_CLI || tipo==ARSTAMPANTE_INTESTAZIONI_AZI)
  prestampabox->attivaTab(1);
 	prestampabox->exec();
 	if(tipo==ARSTAMPANTE_ETICHETTE)        stampaRealEtichette(aux_table,true,1);
 	if(tipo==ARSTAMPANTE_INTESTAZIONI_FOR) stampaRealIntestFor(aux_table,true,1);
 	if(tipo==ARSTAMPANTE_INTESTAZIONI_CLI) stampaRealIntestCli(aux_table,true,1);
 	if(tipo==ARSTAMPANTE_INTESTAZIONI_AZI) stampaRealIntestAzi(aux_table,true,1);
 	if(tipo==ARSTAMPANTE_ETICHETTE)
 	 {
 	  stampaRealEtichette(aux_table,true,1);
 	  prebox->ini_r->setRange(1,sql->getPref(7,"pref_stampante_et").toInt());
 	  prebox->ini_c->setRange(1,sql->getPref(8,"pref_stampante_et").toInt());
 	 }
  else
 	 {
 	  prebox->ini_r->setRange(1,sql->getPref(7,"pref_stampante_in").toInt());
 	  prebox->ini_c->setRange(1,sql->getPref(8,"pref_stampante_in").toInt());
 	 }
 	prebox->setPagina(1);
 	prebox->ridisegna();
 }

void ARStampante::vaiPagina()
 {
 	if(tipo==ARSTAMPANTE_ETICHETTE) stampaRealEtichette(aux_table,true,prebox->getPagina());
 	if(tipo==ARSTAMPANTE_INTESTAZIONI_FOR) stampaRealIntestFor(aux_table,true,prebox->getPagina());
 	if(tipo==ARSTAMPANTE_INTESTAZIONI_CLI) stampaRealIntestCli(aux_table,true,prebox->getPagina());
 	if(tipo==ARSTAMPANTE_INTESTAZIONI_AZI) stampaRealIntestAzi(aux_table,true,prebox->getPagina());
 	prebox->ridisegna();
 }

void ARStampante::primaPagina() { prebox->setPagina(1); vaiPagina(); }

void ARStampante::precPagina() { prebox->addPagina(-1); vaiPagina(); }

void ARStampante::succPagina() { prebox->addPagina(1); vaiPagina(); }

void ARStampante::stampaFattura(QString fatturaID)
 {
 	QString clienteID,logo,azienda,cliente,destinazione,data_fattura;
 	QString tipologia,scontrino,note,numero,html,aux;
  MYSQL_RES *rs,*rs1,*rs3,*rs4;
  MYSQL_ROW row,row1,row3,row4;

  rs=sql->doQueryRS("SELECT * FROM fatture WHERE fatturaID='"+fatturaID+"'");
  if(rs)
   {
 	  if(mysql_num_rows(rs)!=1)
 	   {
 	 	  warnbox->setText(tr("Nulla da stampare!\nStampa non avviata!"));
 	    warnbox->exec();
 	    mysql_free_result(rs);  
 	    return;
 	   }
// 	prendiPref(1);
    row=mysql_fetch_row(rs);
    clienteID=row[1];
    rs1=sql->doQueryRS("SELECT * FROM clienti WHERE clienteID='"+clienteID+"'");
    if(rs1)
     {
  	  if(mysql_num_rows(rs1)!=1)
 	     {
 	 	    warnbox->setText(tr("Errore di stampa, cliente errato!\nStampa non avviata!"));
 	      warnbox->exec();
 	      mysql_free_result(rs1);
 	      return;
 	     }
 	    row1=mysql_fetch_row(rs1);
     }
    else return;
    rs3=sql->doQueryRS("SELECT * FROM aziende WHERE aziendaID='1'");
    if(rs3)
     {
  	  if(mysql_num_rows(rs3)!=1)
 	     {
 	 	    warnbox->setText(tr("Errore di stampa, dati tua azienda errati!\nStampa non avviata!"));
 	      warnbox->exec();
 	      mysql_free_result(rs3);
 	      return;
 	     }
 	    row3=mysql_fetch_row(rs3);
     }
    else return;
    //rs4=sql->doQueryRS("SELECT articoloID,codice,descrizione,prezzo_pub,iva FROM articoli WHERE fatturaID='"+fatturaID+"'");
    
    rs4=sql->doQueryRS("SELECT articoloID,codice,descrizione,prezzo_pub,iva FROM articoli WHERE fatturaID='"+fatturaID+"' UNION SELECT elementoID AS articoloID,articolo AS codice,descrizione,prezzo AS prezzo_pub,iva FROM fatture_elementi_extra WHERE fatturaID='"+fatturaID+"'");
    
    if(!rs4)
     {
 	    warnbox->setText(tr("Errore di stampa, dati relativi agli articoli errati!\nStampa non avviata!"));
      warnbox->exec();
      return;
     }
    mysql_free_result(rs);

    if(stampabox->exec()==QDialog::Accepted)
     {
   	  QTextDocument *foglio = new QTextDocument("",this);

      logo = "<img src=\"pic/logo.png\">";
      aux=row3[1];  azienda="&nbsp;&nbsp;&nbsp;"+aux;
      aux=row3[2];  azienda+="<br>&nbsp;&nbsp;&nbsp;"+aux;
      aux=row3[3];  azienda+="<br>&nbsp;&nbsp;&nbsp;"+aux;
      aux=row3[4];  azienda+=" "+aux;
      aux=row3[5];  azienda+=" ("+aux+")";
      aux=row3[10]; if(aux!="") azienda+="<br>&nbsp;&nbsp;&nbsp;P.IVA "+aux;
      aux=row3[11]; if(aux!="") azienda+="<br>&nbsp;&nbsp;&nbsp;C.F. "+aux;
      aux=row3[7];  azienda+="<br>&nbsp;&nbsp;&nbsp;Tel. "+aux;
      aux=row3[8];  azienda+="<br>&nbsp;&nbsp;&nbsp;Fax "+aux;
      aux=row3[9];  azienda+="<br>&nbsp;&nbsp;&nbsp;Cell. "+aux;
      
      aux=row1[1];  cliente="&nbsp;&nbsp;&nbsp;"+aux;
      aux=row1[2];  cliente+="<br>&nbsp;&nbsp;&nbsp;"+aux;
      aux=row1[3];  cliente+="<br>&nbsp;&nbsp;&nbsp;"+aux;
      aux=row1[4];  cliente+=" "+aux;
      aux=row1[5];  cliente+=" ("+aux+")";
      aux=row1[10]; if(aux!="") cliente+="<br>&nbsp;&nbsp;&nbsp;Partita I.V.A.: "+aux;
                    else { aux=row1[11]; cliente+="<br>&nbsp;&nbsp;&nbsp;Codice Fiscale: "+aux; }

      destinazione=tr("&nbsp;&nbsp;&nbsp;Idem");

      QDate data=QDate::fromString(QString(row[5]),"yyyy-MM-dd");
      data_fattura=data.toString("dd MMMM yyyy");

      numero=row[8];
      tipologia=row[2]; tipologia="&nbsp;&nbsp;&nbsp;"+tipologia;
      scontrino=row[3]; scontrino="&nbsp;&nbsp;&nbsp;"+scontrino;
      note=row[4];
      
	  infobox->setText(tr("Stampa fattura in corso!"));
   	  infobox->exec();
   	  html ="<table width=100% align=left border=1 bordercolor=#000000 cellspacing=0 cellpadding=2>";
   	  html+="<tr><td width=60%>";
   	  html+=""+logo+"<b><br><font size=1>"+azienda+"</font></b><br>";
   	  html+="</td><td width=40%>";
   	  html+="<b>Fattura/Invoice<br>N. "+numero+" del "+data_fattura+"</b><br><hr width=100% size=1><font color=#999999 size=1>Tipologia merce:</font><br><font size=1><b>"+tipologia+"</b></font><hr width=100% size=1><font color=#999999 size=1>Scontrino numero:</font><br><font size=1><b>"+scontrino+"</b></font><br>";
   	  html+="</td></tr>";
   	
   	  html+="<tr><td>";
   	  html+="<font color=#999999 size=1>Cliente:</font><b><br><font size=1>"+cliente+"</font></b><br>";
   	  html+="</td><td>";
   	  html+="&nbsp;";
   	  html+="</td></tr></table>";

      html+="<table width=100% align=left border=1 bordercolor=#000000 cellspacing=0 cellpadding=0>";
 	    html+="<tr><td width=5%><font size=1><b>N.</b></font></td>";
 	    html+="<td width=10%><font size=1><b>Codice</b></font></td>";
    	html+="<td width=15%><font size=1><b>Articolo</b></font></td>";
    	html+="<td width=40%><font size=1><b>Descrizione</b></font></td>";
    	html+="<td width=6%><font size=1><b>Q.t</b></font></td>";

    	html+="<td width=10%><font size=1><b>Prezzo</b></font></td>";
    	html+="<td width=4%><font size=1><b>IVA</b></font></td>";
    	html+="<td width=10%><font size=1><b>Importo</b></font></td>";
    	
    	html+="</tr></table>";
    	int i=0;
    	QString num,articoloID,codice,descrizione,qta,prezzo_uni,iva,_importo,_tot_qta,_imponibile,_tot_iva,_tot_fat;
    	double tot_qta=0,importo=0,imponibile=0,tot_iva=0;
    	double IVA[101];
    	
    	for(int i=0;i<101;i++) IVA[i]=0.0;
      while((row4=mysql_fetch_row(rs4)))
 	     {
 	     	num.sprintf("%d.",i+1);
 	     	articoloID=row4[0]; codice=row4[1]; descrizione=row4[2]; qta="1"; prezzo_uni=row4[3]; iva=row4[4];
 	     	html+="<table width=100% align=left border=1 bordercolor=#000000 cellspacing=0 cellpadding=0>";
 	     	if(i%2==0) html+="<tr>"; else html+="<tr bgcolor=#dddddd>";
 	     	html+="<td width=5%><font size=1>"+num+"</font></td>";
    	  html+="<td width=10%><font size=1>"+articoloID+"</font></td>";
    	  html+="<td width=15%><font size=1>"+codice+"</font></td>";
    	  html+="<td width=40%><font size=1>"+descrizione+"</font></td>";
    	  html+="<td width=6%><font size=1>"+qta+"</font></td>";

    	  importo=qta.toDouble()*prezzo_uni.toDouble(); _importo.sprintf("%.2lf",importo);
    	  imponibile=imponibile+prezzo_uni.toDouble();
    	  tot_qta=tot_qta+qta.toDouble();
    	  tot_iva=tot_iva+(iva.toDouble()/100.0*importo);
    	  IVA[iva.toInt()]=IVA[iva.toInt()]+(iva.toDouble()/100.0*importo);
    	  html+="<td width=10% align=right><font size=1>"+prezzo_uni+"</font></td>";
    	  html+="<td width=4%  align=right><font size=1>"+iva+"%</font></td>";
    	  html+="<td width=10% align=right><font size=1>"+_importo+"</font></td>";

    	  html+="</tr></table>";
    	  i++;
 	     }
 	    mysql_free_result(rs4);
 	    _tot_qta.sprintf("%.0lf",tot_qta);
 	    _imponibile.sprintf("%.2lf",imponibile);
 	    _tot_iva.sprintf("%.2lf",tot_iva);
 	    _tot_fat.sprintf("%.2lf",imponibile+tot_iva);
 	    
      html+="<table width=100% align=left border=1 bordercolor=#000000 cellspacing=0 cellpadding=2>";
    	html+="<tr><td><br><font color=#999999 size=1>Note:</font><br><br><font size=1><b>"+note+"</b></font></td></tr>";
 	    html+="</table>";
 	     
 	    html+="<table width=100% align=left border=1 bordercolor=#000000 cellspacing=0 cellpadding=2>";
   	  html+="<tr><td width=30%>&nbsp;</td>";
   	  
   	  html+="<td width=30%>";
   	  for(int i=0;i<101;i++)
   	   {
   	   	QString _i;
   	    if(IVA[i]>0)
   	     {
   	     	_i.sprintf("%d",i);
   	     	aux.sprintf("%.2lf",IVA[i]);
   	      html+="<font color=#999999 size=1>I.V.A. "+_i+"% ("+mainWin->CURRENCY_NAME+"): </font><b><br><font size=1><div align=right>"+aux+"</div></font></b><br><hr width=100% size=1>";
   	     }
   	   }
   	  html+="</td>";

      html+="<td width=40%>";
   	  html+="<font color=#999999 size=1>Totole Quantit:         </font><b><br><font size=1><div align=right>"+_tot_qta+"</div></font></b><br><hr width=100% size=1>";
   	  html+="<font color=#999999 size=1>Totale Imponibile ("+mainWin->CURRENCY_NAME+"):</font><b><br><font size=1><div align=right>"+_imponibile+"</div></font></b><br><hr width=100% size=1>";
   	  html+="<font color=#999999 size=1>Totale I.V.A. ("+mainWin->CURRENCY_NAME+"):    </font><b><br><font size=1><div align=right>"+_tot_iva+"</div></font></b><br><hr width=100% size=1>";
   	  html+="<font color=#999999 size=1>Totale Fattura ("+mainWin->CURRENCY_NAME+"):   </font><b><br><font size=1><div align=right>"+_tot_fat+"</div></font></b></td>";
  	  html+="</tr></table>";
  	  
   	  foglio->setHtml(html);
   	  foglio->print(stampante);
 	   }
 	 }
 }

void ARStampante::stampaBolla(QString bollaID)
 {
 	QString fornitoreID,indir_resoID,mittente,destinatario,destinazione,data_bolla;
 	QString causale,mezzo,aspetto,colli,peso_kg,porto,inizio,vettore,note,numero,html,aux;
  MYSQL_RES *rs,*rs1,*rs2,*rs3,*rs4;
  MYSQL_ROW row,row1,row2=0,row3,row4;

  rs=sql->doQueryRS("SELECT * FROM bolle WHERE bollaID='"+bollaID+"'");
  if(rs)
   {
 	  if(mysql_num_rows(rs)!=1)
 	   {
 	 	  warnbox->setText(tr("Nulla da stampare!\nStampa non avviata!"));
 	    warnbox->exec();
 	    mysql_free_result(rs);  
 	    return;
 	   }
// 	prendiPref(1);
    row=mysql_fetch_row(rs);
    fornitoreID=row[1];
    indir_resoID=row[2];
    rs1=sql->doQueryRS("SELECT * FROM fornitori WHERE fornitoreID='"+fornitoreID+"'");
    if(rs1)
     {
  	  if(mysql_num_rows(rs1)!=1)
 	     {
 	 	    warnbox->setText(tr("Errore di stampa, fornitore errato!\nStampa non avviata!"));
 	      warnbox->exec();
 	      mysql_free_result(rs1);
 	      return;
 	     }
 	    row1=mysql_fetch_row(rs1);
     }
    else return;
    if(indir_resoID!="0")
     {
      rs2=sql->doQueryRS("SELECT * FROM indirizzi_reso WHERE indir_resoID='"+indir_resoID+"'");
      if(rs2)
       {
  	    if(mysql_num_rows(rs2)!=1)
 	       {
 	 	      warnbox->setText(tr("Errore di stampa, indirizzo di destinazione errato!\nStampa non avviata!"));
 	        warnbox->exec();
 	        mysql_free_result(rs2);
 	        return;
 	       }
 	      row2=mysql_fetch_row(rs2);
       }
      else return;
     }
    rs3=sql->doQueryRS("SELECT * FROM aziende WHERE aziendaID='1'");
    if(rs3)
     {
  	  if(mysql_num_rows(rs3)!=1)
 	     {
 	 	    warnbox->setText(tr("Errore di stampa, dati tua azienda errati!\nStampa non avviata!"));
 	      warnbox->exec();
 	      mysql_free_result(rs3);
 	      return;
 	     }
 	    row3=mysql_fetch_row(rs3);
     }
    else return;
    rs4=sql->doQueryRS("SELECT articoloID,codice,descrizione FROM articoli WHERE bollaID='"+bollaID+"'");
    if(!rs4)
     {
 	    warnbox->setText(tr("Errore di stampa, dati relativi agli articoli errati!\nStampa non avviata!"));
      warnbox->exec();
      return;
     }
    mysql_free_result(rs);

    if(stampabox->exec()==QDialog::Accepted)
     {
   	  QTextDocument *foglio = new QTextDocument("",this);

      aux=row3[1];  mittente="&nbsp;&nbsp;&nbsp;"+aux;
      aux=row3[2];  mittente+="<br>&nbsp;&nbsp;&nbsp;"+aux;
      aux=row3[3];  mittente+="<br>&nbsp;&nbsp;&nbsp;"+aux;
      aux=row3[4];  mittente+=" "+aux;
      aux=row3[5];  mittente+=" ("+aux+")";
      aux=row3[10]; if(aux!="") mittente+="<br>&nbsp;&nbsp;&nbsp;P.IVA "+aux;
      aux=row3[11]; if(aux!="") mittente+="<br>&nbsp;&nbsp;&nbsp;C.F. "+aux;
      aux=row3[7];  mittente+="<br>&nbsp;&nbsp;&nbsp;Tel. "+aux;
      aux=row3[8];  mittente+="<br>&nbsp;&nbsp;&nbsp;Fax "+aux;
      aux=row3[9];  mittente+="<br>&nbsp;&nbsp;&nbsp;Cell. "+aux;
      
      aux=row1[1];  destinatario="&nbsp;&nbsp;&nbsp;"+aux;
      aux=row1[3];  destinatario+="<br>&nbsp;&nbsp;&nbsp;"+aux;
      aux=row1[4];  destinatario+="<br>&nbsp;&nbsp;&nbsp;"+aux;
      aux=row1[5];  destinatario+=" "+aux;
      aux=row1[6];  destinatario+=" ("+aux+")";
      
      if(indir_resoID=="0") { destinazione=tr("&nbsp;&nbsp;&nbsp;Idem"); }
      else
       {
       	aux=row2[2]; if(aux=="") aux=row1[1]; destinazione="&nbsp;&nbsp;&nbsp;"+aux;
        aux=row2[3]; destinazione+="<br>&nbsp;&nbsp;&nbsp;"+aux;
        aux=row2[4]; destinazione+="<br>&nbsp;&nbsp;&nbsp;"+aux;
        aux=row2[5]; destinazione+=" "+aux;
        aux=row2[6]; destinazione+=" ("+aux+")";
       }

      QDate data=QDate::fromString(QString(row[14]),"yyyy-MM-dd");
      data_bolla=data.toString("dd MMMM yyyy");

      numero=row[17];
      causale=row[3]; causale="&nbsp;&nbsp;&nbsp;"+causale;
      mezzo=row[4];
      if(mezzo=="1") mezzo=tr("Mittente");
      if(mezzo=="2") mezzo=tr("Vettore");
      if(mezzo=="3") mezzo=tr("Destinatario");
      mezzo="&nbsp;&nbsp;&nbsp;"+mezzo;
      aspetto=row[5]; aspetto="&nbsp;&nbsp;&nbsp;"+aspetto;
      colli=row[6];   colli="&nbsp;&nbsp;&nbsp;"+colli;
      peso_kg=row[7]; peso_kg="&nbsp;&nbsp;&nbsp;"+peso_kg;
      porto=row[8];
      if(porto=="1") porto=tr("Franco");
      if(porto=="2") porto=tr("Assegnato");
      porto="&nbsp;&nbsp;&nbsp;"+porto;
      vettore="&nbsp;&nbsp;&nbsp;"+QString(row[10])+"<br>&nbsp;&nbsp;&nbsp;"+QString(row[11])+"<br>&nbsp;&nbsp;&nbsp;"+QString(row[12]);
      note=row[13];
	    infobox->setText(tr("Stampa bolla in corso!"));
   	  infobox->exec();
   	  html ="<table width=100% align=left border=1 bordercolor=#000000 cellspacing=0 cellpadding=2>";
   	  html+="<tr><td width=50%>";
   	  html+="<font color=#999999 size=1>Mittente:</font><b><br><font size=1>"+mittente+"</font></b><br>";
   	  html+="</td><td width=50%>";
   	  html+="<b>Documento di trasporto<br>N. "+numero+" del "+data_bolla+"</b><br><hr width=100% size=1><font color=#999999 size=1>Trasporto a mezzo:</font><br><font size=1><b>"+mezzo+"</b></font><hr width=100% size=1><font color=#999999 size=1>Causale del trasporto:</font><br><font size=1><b>"+causale+"</b></font><br>";
   	  html+="</td></tr>";
   	
   	  html+="<tr><td>";
   	  html+="<font color=#999999 size=1>Destinatario:</font><b><br><font size=1>"+destinatario+"</font></b><br>";
   	  html+="</td><td>";
   	  html+="<font color=#999999 size=1>Luogo di destinazione:</font><br><font size=1><b>"+destinazione+"</b></font><br>";
   	  html+="</td></tr></table>";

      html+="<table width=100% align=left border=1 bordercolor=#000000 cellspacing=0 cellpadding=0>";
 	    html+="<tr><td width=8%><font size=1><b>N.</b></font></td>";
 	    html+="<td width=10%><font size=1><b>Codice</b></font></td>";
    	html+="<td width=18%><font size=1><b>Articolo</b></font></td>";
    	html+="<td width=56%><font size=1><b>Descrizione</b></font></td>";
    	html+="<td width=8%><font size=1><b>Q.t</b></font></td>";
    	html+="</tr></table>";
    	int i=0;
    	QString num,articoloID,codice,descrizione,qta;
    	
      while((row4=mysql_fetch_row(rs4)))
 	     {
 	     	num.sprintf("%d.",i+1);
 	     	articoloID=row4[0]; codice=row4[1]; descrizione=row4[2]; qta="1";
 	     	html+="<table width=100% align=left border=1 bordercolor=#000000 cellspacing=0 cellpadding=0>";
 	     	if(i%2==0) html+="<tr>"; else html+="<tr bgcolor=#dddddd>";
 	     	html+="<td width=8%><font size=1>"+num+"</font></td>";
    	  html+="<td width=10%><font size=1>"+articoloID+"</font></td>";
    	  html+="<td width=18%><font size=1>"+codice+"</font></td>";
    	  html+="<td width=56%><font size=1>"+descrizione+"</font></td>";
    	  html+="<td width=8% align=right><font size=1>"+qta+"</font></td>";
    	  html+="</tr></table>";
    	  i++;
 	     }
 	    mysql_free_result(rs4);
 	    
      html+="<table width=100% align=left border=1 bordercolor=#000000 cellspacing=0 cellpadding=2>";
    	html+="<tr><td><br><font color=#999999 size=1>Note:</font><br><br><font size=1><b>"+note+"</b></font></td></tr>";
 	    html+="</table>";
 	     
 	    html+="<table width=100% align=left border=1 bordercolor=#000000 cellspacing=0 cellpadding=2>";
   	  html+="<tr><td width=50%>";
   	  html+="<font color=#999999 size=1>Aspetto esteriore dei beni:</font><b><br><font size=1>"+aspetto+"</font></b><br>";
   	  html+="</td><td width=15%>";
   	  html+="<font color=#999999 size=1>N. colli:</font><b><br><font size=1>"+colli+"</font></b><br>";
   	  html+="</td><td width=15%>";
   	  html+="<font color=#999999 size=1>Peso:</font><b><br><font size=1>"+peso_kg+"</font></b><br>";
   	  html+="</td><td width=20%>";
   	  html+="<font color=#999999 size=1>Porto:</font><b><br><font size=1>"+porto+"</font></b><br></td></tr><tr>";
   	  html+="<td><font color=#999999 size=1>Vettore:</font><b><br><font size=1>"+vettore+"</font></b><br></td>";
   	  html+="<td><font color=#999999 size=1>Data consegna:</font>&nbsp;<br><br><hr width=100% size=1><br><br></td>";
   	  html+="<td><font color=#999999 size=1>Ora consegna:</font>&nbsp;<br><br><hr width=100% size=1><br><br></td>";
   	  html+="<td><font color=#999999 size=1>Firma conducente:</font>&nbsp;<br><br><hr width=100% size=1><br><br></td></tr><tr>";

   	  html+="<td>&nbsp;</td>";
   	  html+="<td><font color=#999999 size=1>Data consegna:</font>&nbsp;<br><br><br></td>";
   	  html+="<td><font color=#999999 size=1>Ora consegna:</font>&nbsp;<br><br><br></td>";
   	  html+="<td><font color=#999999 size=1>Firma destintario:</font>&nbsp;<br><br><br></td>";
  	  html+="</tr></table>";
  	  
   	  foglio->setHtml(html);
   	  foglio->print(stampante);
 	   }
 	 }
 }

void ARStampante::stampaEtichette(QTableWidget * table)
 {
 	bool pre;

  if(sql->getPref(7,"pref_varie")=="1") pre=true; else pre=false;
 	prebox->ini_r->setRange(1,sql->getPref(7,"pref_stampante_et").toInt()); prebox->ini_r->setValue(1);
 	prebox->ini_c->setRange(1,sql->getPref(8,"pref_stampante_et").toInt()); prebox->ini_c->setValue(1);
 	tipo=ARSTAMPANTE_ETICHETTE; aux_table=table;
 	if(mainWin->getARScaricoP()->numRowSel()==0)
 	 {
 	 	warnbox->setText(tr("Nulla da stampare!\nDevi selezionare almeno un articolo.\nStampa non avviata!"));
 	  warnbox->exec();
 	  return;
 	 }
 	if(pre)
 	 {
 	 	prebox->setPagina(1);
 	 	stampaRealEtichette(table,true,1);
 	 	prebox->ridisegna();
    if(!prebox->exec()==QDialog::Accepted) return;
    stampaRealEtichette(table,false);
    infobox->setText(tr("Stampa etichette in corso!"));
    infobox->exec();
   }
  else
   {
   	if(stampabox->exec()==QDialog::Accepted)
     {
      stampaRealEtichette(table,false);
      infobox->setText(tr("Stampa etichette in corso!"));
      infobox->exec();
     }
   }
 }

void ARStampante::stampaRealEtichette(QTableWidget * table, bool pre, int pg)
 {
 	int rr;
 	
 	prendiPref(0);

 	QString bcode,art,descr,misura,prezzo,forn,colore,fondo,tacco,stagione;
 	MYSQL_RES *rs,*rs2;
  MYSQL_ROW row,row2;
 	int dpi,x,y,nr,nc,dim;
 	int act_pg=0;
 	float _1cm,m_destro,m_sinistro,m_alto,m_basso,r_oriz,r_vert,x_offset,y_offset;

  QPainter tela(this);
  //QRect rett = tela.viewport(); //QRect rett = tela.window();
  QRect rett = stampante->paperRect();
  if(pre)
   {
   	prebox->setAnteprima(rett.width(),rett.height());
   	tela.end(); tela.begin(prebox->pic);
   }
  else { tela.end(); tela.begin(stampante); }
  tela.setRenderHint(QPainter::Antialiasing,false);
  tela.setPen(QColor(0,0,0)); 
 	dpi=stampante->resolution();
 	nr=num_r.toInt(); nc=num_c.toInt();
  _1cm = dpi/2.54; // 1 cm in pixel (dpi)
  m_destro=(mdestro.toFloat()/10.0)*_1cm;
  m_sinistro=(msinistro.toFloat()/10.0)*_1cm;
  m_alto=(malto.toFloat()/10.0)*_1cm;
  m_basso=(mbasso.toFloat()/10.0)*_1cm;
  r_oriz=(roriz.toFloat()/10.0)*_1cm;
  r_vert=(rvert.toFloat()/10.0)*_1cm;
  x_offset=((float)rett.width()-(m_destro+m_sinistro))/(float)nc;
  y_offset=((float)rett.height()-(m_alto+m_basso))/(float)nr;
  x=(int)m_destro+(int)r_oriz; y=(int)m_alto+(int)r_vert;

  // Scostamento sulla griglia di stampa
  int ini_x=prebox->ini_c->value()-1;
  int ini_y=prebox->ini_r->value()-1;
  x+=((int)x_offset+(int)r_oriz)*ini_x;
  y+=((int)y_offset+(int)r_vert)*ini_y;
  
  rr=0;
  if(bcodesmall=="0") dim=2; else dim=1;
  for(int r=0;r<table->rowCount();r++)
   {
   	if(table->isItemSelected(table->item(r,0)))
     {
     	rs=sql->doQueryRS("SELECT fornitori.etichetta FROM fornitori,articoli WHERE fornitori.fornitoreID=articoli.fornitoreID AND articoli.articoloID='"+table->item(r,0)->text()+"'");
    	row=mysql_fetch_row(rs);
    	forn=row[0];
   	  mysql_free_result(rs);
      if(campi_vis & 2)
       {
   	    rs2=sql->doQueryRS("SELECT colori.etichetta FROM articoli,colori WHERE colori.coloreID=articoli.coloreID AND articoli.articoloID='"+table->item(r,0)->text()+"'");
   	    row2=mysql_fetch_row(rs2);
   	    if(row2) { colore=row2[0]; colore+=" "; } else { colore=""; }
   	    mysql_free_result(rs2);
   	   }
      if(campi_vis & 4)
       {
	     	rs2=sql->doQueryRS("SELECT fondi.etichetta FROM articoli,fondi WHERE fondi.fondoID=articoli.fondoID AND articoli.articoloID='"+table->item(r,0)->text()+"'");
	     	row2=mysql_fetch_row(rs2);
      	if(row2) { fondo=row2[0]; fondo+=" "; } else { fondo=""; }
      	mysql_free_result(rs2);
       }
      if(campi_vis & 8)
       {
	     	rs2=sql->doQueryRS("SELECT tacchi.etichetta FROM articoli,tacchi WHERE tacchi.taccoID=articoli.taccoID AND articoli.articoloID='"+table->item(r,0)->text()+"'");
	     	row2=mysql_fetch_row(rs2);
      	if(row2) { tacco=row2[0]; tacco+=" "; } else { tacco=""; }
      	mysql_free_result(rs2);
       }
      if(campi_vis & 16)
       {
	     	rs2=sql->doQueryRS("SELECT stagioni.etichetta FROM articoli,stagioni WHERE stagioni.stagioneID=articoli.stagioneID AND articoli.articoloID='"+table->item(r,0)->text()+"'");
	     	row2=mysql_fetch_row(rs2);
      	if(row2) { stagione=row2[0]; stagione+=" "; } else { stagione=""; }
      	mysql_free_result(rs2);
       }

   	  if(forn!="") forn ="("+forn+")";
   	  if(dim==2) // Barcode grandi
   	   {
   	    bcode = table->item(r,0)->text();

  	    if(campi_vis & 1)  art = "Art. "+table->item(r,1)->text(); else art="";
   	    if(campi_vis & 2)  colore+=" ";   else colore="";
   	    if(campi_vis & 4)  fondo+=" ";    else fondo="";
   	    if(campi_vis & 8)  tacco+=" ";    else tacco="";
   	    if(campi_vis & 16) stagione+=" "; else stagione="";

   	    descr  = forn+" "+table->item(r,2)->text();
   	    misura = table->item(r,4)->text();
   	    prezzo = mainWin->CURRENCY_NAME+" "+table->item(r,5)->text();
        drawBcode(bcode,x,y,dim,&tela);
        tela.drawText(x,y+bcodealt+12,descr);
        tela.drawText(x,y+bcodealt+24,art);
        tela.drawText(x+art.length()*6+4*6,y+bcodealt+24,misura);
        tela.drawText(x+art.length()*6+4*6+misura.length()*6+4*6,y+bcodealt+24,prezzo);
        tela.drawText(x+1+art.length()*6+4*6+misura.length()*6+4*6,y+bcodealt+24,prezzo);
        tela.drawText(x,y+bcodealt+36,colore);
        tela.drawText(x+colore.length()*6,y+bcodealt+36,fondo);
        tela.drawText(x+colore.length()*6+fondo.length()*6,y+bcodealt+36,tacco);
        tela.drawText(x+colore.length()*6+fondo.length()*6+tacco.length()*6,y+bcodealt+36,stagione);
       }
      else // Barcode piccoli
       {
   	    bcode = table->item(r,0)->text();

   	    if(campi_vis & 1) art = "Ar. "+table->item(r,1)->text()+" "; else art="";
   	    if(campi_vis & 2)  colore+=" ";   else colore="";
   	    if(campi_vis & 4)  fondo+=" ";    else fondo="";
   	    if(campi_vis & 8)  tacco+=" ";    else tacco="";
   	    if(campi_vis & 16) stagione+=" "; else stagione="";

   	    descr=table->item(r,2)->text();
   	    descr=art+forn+" "+descr;
   	    misura = table->item(r,4)->text();
   	    prezzo = mainWin->CURRENCY_NAME+" "+table->item(r,5)->text();
        drawBcode(bcode,x,y,dim,&tela);
        tela.drawText(x,y+bcodealt+12,descr);
        tela.drawText(x,y+bcodealt+24,misura);
        tela.drawText(x+misura.length()*6+3*6,y+bcodealt+24,prezzo);
        tela.drawText(x+1+misura.length()*6+3*6,y+bcodealt+24,prezzo);
        tela.drawText(x,y+bcodealt+36,colore);
        tela.drawText(x+colore.length()*6,y+bcodealt+36,fondo);
        tela.drawText(x+colore.length()*6+fondo.length()*6,y+bcodealt+36,tacco);
        tela.drawText(x+colore.length()*6+fondo.length()*6+tacco.length()*6,y+bcodealt+36,stagione);
       }
      x+=(int)x_offset+(int)r_oriz;
      if((rr+1+ini_x)%nc==0)      { y+=(int)y_offset+(int)r_vert; x=(int)m_destro+(int)r_oriz; }
      int gri=(nr*nc)-(ini_x+ini_y*nc);
      if((rr+1-gri)%(nr*nc)==0)
       {
        if(!pre) { stampante->newPage(); }
        else     { act_pg++; if(act_pg==pg) { tela.end(); return; } else tela.fillRect(rett,QBrush(QColor(255,255,255))); }
        x=(int)m_destro+(int)r_oriz; y=(int)m_alto+(int)r_vert;
       }
      rr++;
     }
   }
  if(pre) if(pg-1>act_pg) tela.fillRect(rett,QBrush(QColor(255,255,255)));
  tela.end();
 }

void ARStampante::stampaMagazzino(QTableWidget * table)
 {
  if(table->rowCount()==0)
   {
    warnbox->setText(tr("Nulla da stampare!\nStampa non avviata!"));
 	warnbox->exec();
 	return;
   }
// 	prendiPref(1);
  if(stampabox->exec()==QDialog::Accepted)
   {
   	int i=0;
   	QString html,bcode,art,descr,forn,misura,prezzo,iva,totnoiva,totiva,tot;
   	QTextDocument *foglio = new QTextDocument("",this);
   	
	infobox->setText(tr("Stampa magazzino in corso!"));
   	infobox->exec();
   	html="<table width=100% align=left>";
   	if(mainWin->passwordChecked()) html+="<tr bgcolor=#666666><td><font color=#ffffff>Codice a barre:</font></td><td><font color=#ffffff>Articolo:</font></td><td><font color=#ffffff>Descrizione:</font></td><td><font color=#ffffff>Fornitore:</font></td><td><font color=#ffffff>Misura:</font></td><td><font color=#ffffff>Prezzo acq.:</font></td><td><font color=#ffffff>I.V.A.:</font></td></tr>";
   	else                           html+="<tr bgcolor=#666666><td><font color=#ffffff>Codice a barre:</font></td><td><font color=#ffffff>Articolo:</font></td><td><font color=#ffffff>Descrizione:</font></td><td><font color=#ffffff>Fornitore:</font></td><td><font color=#ffffff>Misura:</font></td><td><font color=#ffffff>Prezzo:</font></td></tr>";
 	for(int r=0;r<table->rowCount();r++)
 	 {
   	  bcode  = table->item(r,0)->text();
   	  art    = table->item(r,1)->text();
   	  descr  = table->item(r,2)->text();
   	  forn   = table->item(r,3)->text();
   	  misura = table->item(r,4)->text();
   	  if(mainWin->passwordChecked())
   	   {
		prezzo = table->item(r,11)->text();
		iva    = table->item(r,12)->text()+"%";
	   }
   	  else prezzo = table->item(r,5)->text();
   	  if(i%2==0) html+="<tr bgcolor=#ffffff>"; else html+="<tr bgcolor=#dddddd>";
   	  html+="<td>"+QString(10-bcode.length(),'0')+bcode+"</td>";
   	  html+="<td>"+art+"</td>";
   	  html+="<td>"+descr+"</td>";
   	  html+="<td>"+forn+"</td>";
   	  html+="<td>"+misura+"</td>";
   	  html+="<td align=right>"+prezzo+"</td>";
   	  if(mainWin->passwordChecked()) html+="<td align=right>"+iva+"</td>";
   	  html+="</tr>";
   	  i++;
 	 }
 	if(mainWin->passwordChecked())
 	 {
 	  totnoiva=mainWin->getARScaricoP()->infobar->txt3->text();
 	  totiva=mainWin->getARScaricoP()->infobar->txt4->text();
 	  tot=mainWin->getARScaricoP()->infobar->txt5->text();
      if(mainWin->getARScaricoP()->listaMostra->itemData(mainWin->getARScaricoP()->listaMostra->currentIndex()) == ARSTATO_GIACENZA_ALLA_DATA)
       {
 	    html+="<tr bgcolor=#666666><td colspan=3><font color=#ffffff>Giacenze alla data:</font></td><td colspan=2 align=right><font color=#ffffff>Totale ("+mainWin->CURRENCY_NAME+"):</font></td>       <td align=right><font color=#ffffff>"+totnoiva+"</font></td><td> </td></tr>";
 	    html+="<tr bgcolor=#666666><td colspan=3><font color=#ffffff>"+mainWin->getARScaricoP()->to->date().toString("dd-MMM-yyyy")+"</font></td><td colspan=2 align=right><font color=#ffffff>Totale I.V.A. ("+mainWin->CURRENCY_NAME+"):</font></td><td align=right><font color=#ffffff>"+totiva+"</font></td><td> </td></tr>";
	   }
      else
       {
 	    html+="<tr bgcolor=#666666><td colspan=3> </td><td colspan=2 align=right><font color=#ffffff>Totale ("+mainWin->CURRENCY_NAME+"):</font></td>       <td align=right><font color=#ffffff>"+totnoiva+"</font></td><td> </td></tr>";
 	    html+="<tr bgcolor=#666666><td colspan=3> </td><td colspan=2 align=right><font color=#ffffff>Totale I.V.A. ("+mainWin->CURRENCY_NAME+"):</font></td><td align=right><font color=#ffffff>"+totiva+"</font></td><td> </td></tr>";
	   }
 	  html+="<tr bgcolor=#666666><td colspan=3> </td><td colspan=2 align=right><font color=#ffffff>Totale ivato ("+mainWin->CURRENCY_NAME+"):</font></td> <td align=right><font color=#ffffff>"+tot+"</font></td><td> </td></tr>";
 	 }
 	else
 	 {
      if(mainWin->getARScaricoP()->listaMostra->itemData(mainWin->getARScaricoP()->listaMostra->currentIndex()) == ARSTATO_GIACENZA_ALLA_DATA)
       {
        tot=mainWin->getARScaricoP()->infobar->txt2->text();
 	    html+="<tr bgcolor=#666666><td colspan=3><font color=#ffffff>Giacenze alla data: "+mainWin->getARScaricoP()->to->date().toString("dd-MMM-yyyy")+"</font></td><td align=right><font color=#ffffff>Totale al pubblico ("+mainWin->CURRENCY_NAME+"):</font></td><td colspan=2 align=right><font color=#ffffff>"+tot+"</font></td></tr>";
	   }
	  else
       {
        tot=mainWin->getARScaricoP()->infobar->txt2->text();
 	    html+="<tr bgcolor=#666666><td colspan=3> </td><td align=right><font color=#ffffff>Totale al pubblico ("+mainWin->CURRENCY_NAME+"):</font></td><td colspan=2 align=right><font color=#ffffff>"+tot+"</font></td></tr>";
	   }
	 }
 	html+="</table>";
   	foglio->setHtml(html);
   	foglio->print(stampante);
   }
 }

void ARStampante::stampaIntestAzi(QTableWidget *table)
 {
 	MYSQL_RES *rs;
	QString aziendaID="1";
 	bool pre;

  if(sql->getPref(7,"pref_varie")=="1") pre=true; else pre=false;
 	prebox->ini_r->setRange(1,sql->getPref(7,"pref_stampante_in").toInt()); prebox->ini_r->setValue(1);
 	prebox->ini_c->setRange(1,sql->getPref(8,"pref_stampante_in").toInt()); prebox->ini_c->setValue(1);  
 	tipo=ARSTAMPANTE_INTESTAZIONI_AZI; aux_table=table;
  rs=sql->doQueryRS("SELECT * FROM aziende WHERE aziendaID='"+aziendaID+"'");
  if(mysql_num_rows(rs)!=1)
   {
    warnbox->setText(tr("Nulla da stampare!\nStampa non avviata!"));
    warnbox->exec();
    mysql_free_result(rs);
    return;
   }
 	if(pre)
 	 {
 	 	prebox->setPagina(1);
 	 	stampaRealIntestAzi(table,true,1);
 	 	prebox->ridisegna();
    if(!prebox->exec()==QDialog::Accepted) return;
    stampaRealIntestAzi(table,false);
    infobox->setText(tr("Stampa intestazioni della tua azienda in corso!"));
    infobox->exec();
   }
  else
   {
   	if(stampabox->exec()==QDialog::Accepted)
     {
      stampaRealIntestAzi(table,false);
      infobox->setText(tr("Stampa intestazioni della tua azienda in corso!"));
      infobox->exec();
     }
   }
 }

void ARStampante::stampaRealIntestAzi(QTableWidget *table, bool pre, int pg)
 {
 	prendiPref(1);
  MYSQL_RES *rs;
  MYSQL_ROW row;
  QString aziendaID="1";
  QString nome,indr,cap,citta,prov,piva,email,sitoweb;

  rs=sql->doQueryRS("SELECT * FROM aziende WHERE aziendaID='"+aziendaID+"'");
  if(mysql_num_rows(rs)!=1)
   {
    warnbox->setText(tr("Nulla da stampare!\nStampa non avviata!"));
    warnbox->exec();
    mysql_free_result(rs);
   }
  else
   {
    row=mysql_fetch_row(rs);
    mysql_free_result(rs);
    int dpi,x,y,nr,nc;
    int act_pg=0;
    float _1cm,m_destro,m_sinistro,m_alto,m_basso,r_oriz,r_vert,x_offset,y_offset;

    QPainter tela(this);
    QRect rett = stampante->paperRect();
    if(pre)
     {
  	  prebox->setAnteprima(rett.width(),rett.height());
   	  tela.end(); tela.begin(prebox->pic);
     }
    else { tela.end(); tela.begin(stampante); }
    tela.setRenderHint(QPainter::Antialiasing,false);
    tela.setPen(QColor(0,0,0));
    dpi=stampante->resolution();
    nr=num_r.toInt(); nc=num_c.toInt();
    _1cm = dpi/2.54; // 1 cm in pixel (dpi)
    m_destro=(mdestro.toFloat()/10.0)*_1cm;
    m_sinistro=(msinistro.toFloat()/10.0)*_1cm;
    m_alto=(malto.toFloat()/10.0)*_1cm;
    m_basso=(mbasso.toFloat()/10.0)*_1cm;
    r_oriz=(roriz.toFloat()/10.0)*_1cm;
    r_vert=(rvert.toFloat()/10.0)*_1cm;
    x_offset=((float)rett.width()-(m_destro+m_sinistro))/(float)nc;
    y_offset=((float)rett.height()-(m_alto+m_basso))/(float)nr;
    x=(int)m_destro+(int)r_oriz; y=(int)m_alto+(int)r_vert;

    nome=row[1];
    indr=row[2];
    cap=row[3]; citta=row[4]; prov=row[5];
    email=row[12]; sitoweb=row[13];
    // tel->setText(row[7]); fax->setText(row[8]); cell->setText(row[9]);
    piva=row[10];
 	  nome    = "  "+nome;
 	  indr    = "  "+indr;
 	  citta   = "  "+cap+" "+citta+" ("+prov+")";
	  piva    = "  p. iva "+piva;
 	  email   = "  email  "+email;
 	  sitoweb = "  web    "+sitoweb;
 	  
    // Scostamento sulla griglia di stampa
    int ini_x=prebox->ini_c->value()-1;
    int ini_y=prebox->ini_r->value()-1;
    x+=((int)x_offset+(int)r_oriz)*ini_x;
    y+=((int)y_offset+(int)r_vert)*ini_y;

    //for(int r=0;r<nr*nc;r++)
    for(int r=0;r<nr*nc-(ini_x+ini_y*nc);r++)
     {
 	    tela.drawText(x,y+12 ,nome);
      tela.drawText(x,y+24 ,indr);
      tela.drawText(x,y+36 ,citta);
      tela.drawText(x,y+48,piva);
      tela.drawText(x,y+60,email);
      tela.drawText(x,y+72,sitoweb);
      //tela.drawText(x+art.length()*6+4*6,y+84,misura);
      //tela.drawText(x+art.length()*6+4*6+misura.length()*6+4*6,y+84,prezzo);
      x+=(int)x_offset+(int)r_oriz;
      if((r+1+ini_x)%nc==0)      { y+=(int)y_offset+(int)r_vert; x=(int)m_destro+(int)r_oriz; }
      /*int gri=(nr*nc)-(ini_x+ini_y*nc);
      if((r+1-gri)%(nr*nc)==0)
       {
        if(!pre) { stampante->newPage(); }
        else     { act_pg++; if(act_pg==pg) { tela.end(); return; } else tela.fillRect(rett,QBrush(QColor(255,255,255))); }
        x=(int)m_destro+(int)r_oriz; y=(int)m_alto+(int)r_vert; 
       }*/
     }
    if(pre) if(pg-1>act_pg) tela.fillRect(rett,QBrush(QColor(255,255,255)));
    tela.end();
   }
 }

void ARStampante::stampaIntestFor(QTableWidget * table)
 {
 	bool pre;

  if(sql->getPref(7,"pref_varie")=="1") pre=true; else pre=false;
 	prebox->ini_r->setRange(1,sql->getPref(7,"pref_stampante_in").toInt()); prebox->ini_r->setValue(1);
 	prebox->ini_c->setRange(1,sql->getPref(8,"pref_stampante_in").toInt()); prebox->ini_c->setValue(1);  
 	tipo=ARSTAMPANTE_INTESTAZIONI_FOR; aux_table=table;
 	if(table->rowCount()==0)
 	 {
 	 	warnbox->setText(tr("Nulla da stampare!\nStampa non avviata!"));
 	  warnbox->exec();
 	  return;
 	 }
 	if(pre)
 	 {
 	 	prebox->setPagina(1);
 	 	stampaRealIntestFor(table,true,1);
 	 	prebox->ridisegna();
    if(!prebox->exec()==QDialog::Accepted) return;
    stampaRealIntestFor(table,false);
    infobox->setText(tr("Stampa intestazioni dei fornitori in corso!"));
   	infobox->exec();
   }
  else
   {
   	if(stampabox->exec()==QDialog::Accepted)
     {
      stampaRealIntestFor(table,false);
      infobox->setText(tr("Stampa intestazioni dei fornitori in corso!"));
      infobox->exec();
     }
   }
 }

void ARStampante::stampaRealIntestFor(QTableWidget * table, bool pre, int pg)
 {
 	prendiPref(1);
 	QString nome,indr,citta;
 	int dpi,x,y,nr,nc;
 	int act_pg=0;
 	float _1cm,m_destro,m_sinistro,m_alto,m_basso,r_oriz,r_vert,x_offset,y_offset;

  QPainter tela(this);
  QRect rett = stampante->paperRect();
  if(pre)
   {
  	prebox->setAnteprima(rett.width(),rett.height());
   	tela.end(); tela.begin(prebox->pic);
   }
  else { tela.end(); tela.begin(stampante); }
  tela.setRenderHint(QPainter::Antialiasing,false);
  tela.setPen(QColor(0,0,0));
 	dpi=stampante->resolution();
 	nr=num_r.toInt(); nc=num_c.toInt();
  _1cm = dpi/2.54; // 1 cm in pixel (dpi)
  m_destro=(mdestro.toFloat()/10.0)*_1cm;
  m_sinistro=(msinistro.toFloat()/10.0)*_1cm;
  m_alto=(malto.toFloat()/10.0)*_1cm;
  m_basso=(mbasso.toFloat()/10.0)*_1cm;
  r_oriz=(roriz.toFloat()/10.0)*_1cm;
  r_vert=(rvert.toFloat()/10.0)*_1cm;
  x_offset=((float)rett.width()-(m_destro+m_sinistro))/(float)nc;
  y_offset=((float)rett.height()-(m_alto+m_basso))/(float)nr;
  x=(int)m_destro+(int)r_oriz; y=(int)m_alto+(int)r_vert;

 	// Scostamento sulla griglia di stampa
  int ini_x=prebox->ini_c->value()-1;
  int ini_y=prebox->ini_r->value()-1;
  x+=((int)x_offset+(int)r_oriz)*ini_x;
  y+=((int)y_offset+(int)r_vert)*ini_y;  

  for(int r=0;r<table->rowCount();r++)
   {
 	  nome  = "  "+table->item(r,1)->text();
 	  indr  = "  "+table->item(r,15)->text();
 	  citta = "  "+table->item(r,16)->text();
 	  tela.drawText(x,y+12 ,titfor);
    tela.drawText(x,y+24 ,nome);
    tela.drawText(x,y+36 ,indr);
    tela.drawText(x,y+48,citta);
    //tela.drawText(x+art.length()*6+4*6,y+84,misura);
    //tela.drawText(x+art.length()*6+4*6+misura.length()*6+4*6,y+84,prezzo);
    x+=(int)x_offset+(int)r_oriz;
    if((r+1+ini_x)%nc==0)      { y+=(int)y_offset+(int)r_vert; x=(int)m_destro+(int)r_oriz; }
    int gri=(nr*nc)-(ini_x+ini_y*nc);
    if((r+1-gri)%(nr*nc)==0) 
     { 
      if(!pre) { stampante->newPage(); }
      else     { act_pg++; if(act_pg==pg) { tela.end(); return; } else tela.fillRect(rett,QBrush(QColor(255,255,255))); }
      x=(int)m_destro+(int)r_oriz; y=(int)m_alto+(int)r_vert;
     }
   }
  if(pre) if(pg-1>act_pg) tela.fillRect(rett,QBrush(QColor(255,255,255)));
  tela.end();
 }

void ARStampante::stampaFornitori(QTableWidget * table)
 {
  if(table->rowCount()==0)
   {
    warnbox->setText(tr("Nulla da stampare!\nStampa non avviata!"));
    warnbox->exec();
    return;
   }
// 	prendiPref(1);
  if(stampabox->exec()==QDialog::Accepted)
   {
   	int i=0;
   	QString html,nome,indr,naz,tel,fax,cell,piva,email,web,banca,abi,cab,cin,cc;
   	QTextDocument *foglio = new QTextDocument("",this);
   	
    infobox->setText(tr("Stampa lista fornitori in corso!"));
   	infobox->exec();
   	html="<table width=100% align=left>";
   	html+="<tr bgcolor=#666666><td><font color=#ffffff>Nome:</font></td><td><font color=#ffffff>Indirizzo:</font></td><td><font color=#ffffff>Tel./Fax/Cell:</font></td><td><font color=#ffffff>PIva/C.F.:</font></td><td><font color=#ffffff>Dati bancari:</font></td><td><font color=#ffffff>Dati web:</font></td></tr>";
 	  for(int r=0;r<table->rowCount();r++)
 	   {
   	  nome  = table->item(r,1)->text();
   	  indr  = table->item(r,2)->text();
   	  naz   = table->item(r,3)->text();
   	  tel   = table->item(r,4)->text();
   	  fax   = table->item(r,5)->text();
   	  cell  = table->item(r,6)->text();
   	  piva  = table->item(r,7)->text();
   	  email = table->item(r,8)->text();
   	  web   = table->item(r,9)->text();
   	  banca = table->item(r,10)->text();
   	  abi   = table->item(r,11)->text();
   	  cab   = table->item(r,12)->text();
   	  cin   = table->item(r,13)->text();
   	  cc    = table->item(r,14)->text();
   	  if(i%2==0) html+="<tr bgcolor=#ffffff>"; else html+="<tr bgcolor=#dddddd>";
   	  html+="<td>"+nome+"</td>";
   	  if(naz!="") html+="<td>"+indr+" ["+naz+"]</td>"; else html+="<td>"+indr+"</td>";
   	  html+="<td>";
   	   if(tel!="") html+="Tel. "+tel;
   	   if(fax!="") html+="<br>Fax "+fax;
   	   if(tel!="") html+="<br>Cell "+cell;
   	  html+="</td>";
   	  html+="<td>"+piva+"</td>";
   	  html+="<td>";
   	   html+=banca;
   	   if(abi!="") html+="<br>ABI: "+abi;
   	   if(cab!="") html+="<br>CAB: "+cab;
   	   if(cin!="") html+="<br>CIN: "+cin;
   	   if(cc!="")  html+="<br>C C:  "+cc;
   	  html+="</td>";
   	  html+="<td>";
   	   if(email!="") html+="Email: "+email;
   	   if(web!="") html+="<br>Sito web: "+web;
   	  html+="</td>";
   	  i++;
 	   }
 	  html+="</table>";
   	foglio->setHtml(html);
   	foglio->print(stampante);
 	 }
 }

void ARStampante::stampaIntestCli(QTableWidget * table)
 {
 	bool pre;

  if(sql->getPref(7,"pref_varie")=="1") pre=true; else pre=false;
 	prebox->ini_r->setRange(1,sql->getPref(7,"pref_stampante_in").toInt()); prebox->ini_r->setValue(1);
 	prebox->ini_c->setRange(1,sql->getPref(8,"pref_stampante_in").toInt()); prebox->ini_c->setValue(1);
 	tipo=ARSTAMPANTE_INTESTAZIONI_CLI; aux_table=table;
 	if(table->rowCount()==0)
 	 {
 	 	warnbox->setText(tr("Nulla da stampare!\nStampa non avviata!"));
 	  warnbox->exec();
 	  return;
 	 }
 	if(pre)
 	 {
 	 	prebox->setPagina(1);
 	 	stampaRealIntestCli(table,true,1);
 	 	prebox->ridisegna();
    if(!prebox->exec()==QDialog::Accepted) return;
    stampaRealIntestCli(table,false);
   	infobox->setText(tr("Stampa intestazioni dei clienti in corso!"));
 	  infobox->exec();
   }
  else
   {
   	if(stampabox->exec()==QDialog::Accepted)
     {
      stampaRealIntestCli(table,false);
    	infobox->setText(tr("Stampa intestazioni dei clienti in corso!"));
 	    infobox->exec();
     }
   }
 }

void ARStampante::stampaRealIntestCli(QTableWidget * table, bool pre, int pg)
 {
 	prendiPref(1);
 	QString nome,indr,citta;
 	int dpi,x,y,nr,nc;
 	int act_pg=0;
 	float _1cm,m_destro,m_sinistro,m_alto,m_basso,r_oriz,r_vert,x_offset,y_offset;

  QPainter tela(this);
  QRect rett = stampante->paperRect();
  if(pre)
   {
  	prebox->setAnteprima(rett.width(),rett.height());
   	tela.end(); tela.begin(prebox->pic);
   }
  else { tela.end(); tela.begin(stampante); }
  tela.setRenderHint(QPainter::Antialiasing,false);
  tela.setPen(QColor(0,0,0));
 	dpi=stampante->resolution();
 	nr=num_r.toInt(); nc=num_c.toInt();
  _1cm = dpi/2.54; // 1 cm in pixel (dpi)
  m_destro=(mdestro.toFloat()/10.0)*_1cm;
  m_sinistro=(msinistro.toFloat()/10.0)*_1cm;
  m_alto=(malto.toFloat()/10.0)*_1cm;
  m_basso=(mbasso.toFloat()/10.0)*_1cm;
  r_oriz=(roriz.toFloat()/10.0)*_1cm;
  r_vert=(rvert.toFloat()/10.0)*_1cm;
  x_offset=((float)rett.width()-(m_destro+m_sinistro))/(float)nc;
  y_offset=((float)rett.height()-(m_alto+m_basso))/(float)nr;
  x=(int)m_destro+(int)r_oriz; y=(int)m_alto+(int)r_vert;
  
  // Scostamento sulla griglia di stampa
  int ini_x=prebox->ini_c->value()-1;
  int ini_y=prebox->ini_r->value()-1;
  x+=((int)x_offset+(int)r_oriz)*ini_x;
  y+=((int)y_offset+(int)r_vert)*ini_y;

  for(int r=0;r<table->rowCount();r++)
   {
 	  nome  = "  "+table->item(r,1)->text();
 	  indr  = "  "+table->item(r,11)->text();
 	  citta = "  "+table->item(r,12)->text();
 	  tela.drawText(x,y+12 ,titcli);
    tela.drawText(x,y+24 ,nome);
    tela.drawText(x,y+36 ,indr);
    tela.drawText(x,y+48,citta);
    //tela.drawText(x+art.length()*6+4*6,y+84,misura);
    //tela.drawText(x+art.length()*6+4*6+misura.length()*6+4*6,y+84,prezzo);
    x+=(int)x_offset+(int)r_oriz;
    if((r+1+ini_x)%nc==0)      { y+=(int)y_offset+(int)r_vert; x=(int)m_destro+(int)r_oriz; }
    int gri=(nr*nc)-(ini_x+ini_y*nc);
    if((r+1-gri)%(nr*nc)==0)
     {
      if(!pre) { stampante->newPage(); }
      else     { act_pg++; if(act_pg==pg) { tela.end(); return; } else tela.fillRect(rett,QBrush(QColor(255,255,255))); }
      x=(int)m_destro+(int)r_oriz; y=(int)m_alto+(int)r_vert; 
     }
   }
  if(pre) if(pg-1>act_pg) tela.fillRect(rett,QBrush(QColor(255,255,255)));
  tela.end();
 }

void ARStampante::stampaClienti(QTableWidget * table)
 {
  if(table->rowCount()==0)
   {
    warnbox->setText(tr("Nulla da stampare!\nStampa non avviata!"));
    warnbox->exec();
    return;
   }
// 	prendiPref(1);
  if(stampabox->exec()==QDialog::Accepted)
   {
   	int i=0;
   	QString html,nome,indr,naz,tel,fax,cell,piva,email,web,banca,abi,cab,cin,cc;
   	QTextDocument *foglio = new QTextDocument("",this);
   	
    infobox->setText(tr("Stampa lista clienti in corso!"));
   	infobox->exec();
   	html="<table width=100% align=left>";
   	html+="<tr bgcolor=#666666><td><font color=#ffffff>Nome:</font></td><td><font color=#ffffff>Indirizzo:</font></td><td><font color=#ffffff>Tel./Fax/Cell:</font></td><td><font color=#ffffff>PIva/C.F.:</font></td><td><font color=#ffffff>Dati web:</font></td></tr>";
    for(int r=0;r<table->rowCount();r++)
     {
  	  nome  = table->item(r,1)->text();
   	  indr  = table->item(r,2)->text();
   	  naz   = table->item(r,3)->text();
   	  tel   = table->item(r,4)->text();
   	  fax   = table->item(r,5)->text();
   	  cell  = table->item(r,6)->text();
   	  piva  = table->item(r,7)->text();
   	  email = table->item(r,8)->text();
   	  if(i%2==0) html+="<tr bgcolor=#ffffff>"; else html+="<tr bgcolor=#dddddd>";
   	  html+="<td>"+nome+"</td>";
   	  if(naz!="") html+="<td>"+indr+" ["+naz+"]</td>"; else html+="<td>"+indr+"</td>";
   	  html+="<td>";
      if(tel!="") html+="Tel. "+tel;
   	  if(fax!="") html+="<br>Fax "+fax;
   	  if(tel!="") html+="<br>Cell "+cell;
   	  html+="</td>";
   	  html+="<td>"+piva+"</td>";
   	  html+="<td>";
      if(email!="") html+="Email: "+email;
   	  html+="</td>";
   	  i++;
     }
 	html+="</table>";
   	foglio->setHtml(html);
   	foglio->print(stampante);
   }
 }

void ARStampante::stampaServizi(QTableWidget * table)
 {
 	if(table->rowCount()==0)
 	 {
 	 	warnbox->setText(tr("Nulla da stampare!\nStampa non avviata!"));
 	  warnbox->exec();
 	  return;
 	 }
// 	prendiPref(1);
  if(stampabox->exec()==QDialog::Accepted)
   {
   	int i=0;
   	QString html,codice,descrizione,prezzo,iva,ivato,note;
    QString ts=sql->timeStamp();
    QString data=sql->data(ts,false);
    QString ora=sql->ora(ts); 	
   	QTextDocument *foglio = new QTextDocument("",this);

	  infobox->setText(tr("Stampa lista Servizi in corso!"));
   	infobox->exec();
   	html="<table width=100% align=left>";
    html+="<tr bgcolor=#000000><td colspan=6><font color=#ffffff>Lista servizi alla data: "+data+" ore "+ora+" </font></td></tr>";
   	html+="<tr bgcolor=#666666><td><font color=#ffffff>Codice: </font></td><td><font color=#ffffff>Descrizione: </font></td><td><font color=#ffffff>Prezzo: </font></td><td><font color=#ffffff>IVA: </font></td><td><font color=#ffffff>Prezzo ivato: </font></td><td><font color=#ffffff>Note: </font></td></tr>";
 	  for(int r=0;r<table->rowCount();r++)
 	   {
   	  codice      = table->item(r,1)->text();
   	  descrizione = table->item(r,2)->text();
   	  prezzo      = table->item(r,3)->text();
   	  iva         = table->item(r,4)->text();
   	  ivato       = table->item(r,5)->text();
   	  note        = table->item(r,6)->text();
   	  if(i%2==0) html+="<tr bgcolor=#ffffff>"; else html+="<tr bgcolor=#dddddd>";
   	  html+="<td>"+codice+"</td>";
   	  html+="<td>"+descrizione+"</td>";
   	  html+="<td>"+prezzo+"</td>";
   	  html+="<td>"+iva+"%</td>";
   	  html+="<td>"+ivato+"</td>";
   	  html+="<td>"+note+"</td>";
   	  i++;
 	   }
 	  html+="</table>";
   	foglio->setHtml(html);
   	foglio->print(stampante);
 	 }
 }

void ARStampante::drawBcode(QString str, int x, int y, int s, QPainter *canv)
 {
 	QString sub;
 	int v,offset;

  if(str.length()>10) return; // Max 10 cifre
  if(str.length()<10) str.prepend(QString(10-str.length(),'0')); 
  sub=""; v=0; offset=0;
 	drawBcodeChar(BCLUT[107],x,y,s,&offset,canv); // Blankzone (iniziale)
 	drawBcodeChar(BCLUT[105],x,y,s,&offset,canv); // Start-C (solo numeri)
 	for(int i=0; i<str.length();i+=2)             // Dati (dalla tabella)
 	 {
	 	sub=str.at(i); sub+=str.at(i+1);
	 	v=sub.toInt();
 	 	drawBcodeChar(BCLUT[v],x,y,s,&offset,canv);
 	 }
 	drawBcodeChar(BCLUT[checkBcode(str)],x,y,s,&offset,canv); // Checksum
 	drawBcodeChar(BCLUT[106],x,y,s,&offset,canv); // Stop
 	drawBcodeChar(BCLUT[107],x,y,s,&offset,canv); // Blankzone (finale)
 	// Testo sotto il codice
 	QString aux;
 	aux.sprintf("-%d",checkBcode(str)); str.append(aux);
 	canv->fillRect(x+offset/2-str.length()*6,y+bcodealt-12,str.length()*12,12,QBrush(QColor(255,255,255)));
 	canv->drawText(x+offset/2-str.length()*3,y+bcodealt,str);
 }

void ARStampante::drawBcodeChar(QString str, int x, int y, int s, int *offset, QPainter *canv)
 {
 	for(int i=0;i<str.length();i++)
 	 {
 	 	if(str.at(i)=='1')
 	 	 {
 	 	 	canv->fillRect(x+*offset,y,s,bcodealt,QBrush(QColor(0,0,0)));
 	   }
 	  *offset=*offset+s;
 	 }
 }

int ARStampante::checkBcode(QString str)
 {
 	QString sub;
 	int t,m;
 	
 	sub=""; t=105; m=1;
 	for(int i=0;i<=str.length();i+=2)
 	 {
 	 	sub=str.at(i); sub+=str.at(i+1);
 	  t+=(sub.toInt()*m);
 	  m++;
 	 }
 	return t%103;
 }

void ARStampante::initBcodeLUT()
 {
 	// Uso la codifica C (solo numerica)
 	BCLUT[0] ="11011001100"; BCLUT[53]="11011101110"; BCLUT[100]="10111101110";   // Code B
 	BCLUT[1] ="11001101100"; BCLUT[54]="11101011000"; BCLUT[101]="11101011110";   // Code A
 	BCLUT[2] ="11001100110"; BCLUT[55]="11101000110"; BCLUT[102]="11110101110";   // FNC1
 	BCLUT[3] ="10010011000"; BCLUT[56]="11100010110"; BCLUT[103]="11010000100";   // Start A
 	BCLUT[4] ="10010001100"; BCLUT[57]="11101101000"; BCLUT[104]="11010010000";   // Start B
 	BCLUT[5] ="10001001100"; BCLUT[58]="11101100010"; BCLUT[105]="11010011100";   // Start C
 	BCLUT[6] ="10011001000"; BCLUT[59]="11100011010"; BCLUT[106]="1100011101011"; // Stop
 	BCLUT[7] ="10011000100"; BCLUT[60]="11101111010"; BCLUT[107]="00000000000";   // Blankzone
 	BCLUT[8] ="10001100100"; BCLUT[61]="11001000010";
 	BCLUT[9] ="11001001000"; BCLUT[62]="11110001010";
 	BCLUT[10]="11001000100"; BCLUT[63]="10100110000";
 	BCLUT[11]="11000100100"; BCLUT[64]="10100001100";
 	BCLUT[12]="10110011100"; BCLUT[65]="10010110000";
 	BCLUT[13]="10011011100"; BCLUT[66]="10010000110";
 	BCLUT[14]="10011001110"; BCLUT[67]="10000101100";
 	BCLUT[15]="10111001100"; BCLUT[68]="10000100110";
 	BCLUT[16]="10011101100"; BCLUT[69]="10110010000";
 	BCLUT[17]="10011100110"; BCLUT[70]="10110000100";
 	BCLUT[18]="11001110010"; BCLUT[71]="10011010000";
 	BCLUT[19]="11001011100"; BCLUT[72]="10011000010";
 	BCLUT[20]="11001001110"; BCLUT[73]="10000110100";
 	BCLUT[21]="11011100100"; BCLUT[74]="10000110010";
 	BCLUT[22]="11001110100"; BCLUT[75]="11000010010";
 	BCLUT[23]="11101101110"; BCLUT[76]="11001010000";
 	BCLUT[24]="11101001100"; BCLUT[77]="11110111010";
 	BCLUT[25]="11100101100"; BCLUT[78]="11000010100";
 	BCLUT[26]="11100100110"; BCLUT[79]="10001111010";
 	BCLUT[27]="11101100100"; BCLUT[80]="10100111100";
 	BCLUT[28]="11100110100"; BCLUT[81]="10010111100";
 	BCLUT[29]="11100110010"; BCLUT[82]="10010011110";
 	BCLUT[30]="11011011000"; BCLUT[83]="10111100100";
 	BCLUT[31]="11011000110"; BCLUT[84]="10011110100";
 	BCLUT[32]="11000110110"; BCLUT[85]="10011110010";
 	BCLUT[33]="10100011000"; BCLUT[86]="11110100100";
 	BCLUT[34]="10001011000"; BCLUT[87]="11110010100";
 	BCLUT[35]="10001000110"; BCLUT[88]="11110010010";
 	BCLUT[36]="10110001000"; BCLUT[89]="11011011110";
 	BCLUT[37]="10001101000"; BCLUT[90]="11011110110";
 	BCLUT[38]="10001100010"; BCLUT[91]="11110110110";
 	BCLUT[39]="11010001000"; BCLUT[92]="10101111000";
 	BCLUT[40]="11000101000"; BCLUT[93]="10100011110";
 	BCLUT[41]="11000100010"; BCLUT[94]="10001011110";
 	BCLUT[42]="10110111000"; BCLUT[95]="10111101000";
 	BCLUT[43]="10110001110"; BCLUT[96]="10111100010";
 	BCLUT[44]="10001101110"; BCLUT[97]="11110101000";
 	BCLUT[45]="10111011000"; BCLUT[98]="11110100010";
 	BCLUT[46]="10111000110"; BCLUT[99]="10111011110";
 	BCLUT[47]="10001110110";
 	BCLUT[48]="11101110110";
 	BCLUT[49]="11010001110";
 	BCLUT[50]="11000101110";
 	BCLUT[51]="11011101000";
 	BCLUT[52]="11011100010";
 }
